@@ -158,15 +158,22 @@ Java_org_krita_android_JNIWrappers_saveState(JNIEnv* /*env*/,
     kritarc.setValue("canvasState", "OPENGL_SUCCESS");
 }
 
-extern "C" JNIEXPORT void JNICALL
+extern "C" JNIEXPORT jboolean JNICALL
 Java_org_krita_android_JNIWrappers_exitFullScreen(JNIEnv* /*env*/,
                                                   jobject /*obj*/,
                                                   jint    /*n*/)
 {
-    if (!KisPart::exists()) return;
+    if (!KisPart::exists()) {
+        return false;
+    }
 
     KisMainWindow *mainWindow = KisPart::instance()->currentMainwindow();
-    mainWindow->viewFullscreen(false);
+    if (mainWindow) {
+        mainWindow->viewFullscreen(false);
+        return true;
+    } else {
+        return false;
+    }
 }
 
 extern "C" JNIEXPORT void JNICALL
@@ -258,7 +265,7 @@ extern "C" int main(int argc, char **argv)
             QCoreApplication::setAttribute(Qt::AA_EnableHighDpiScaling);
         }
 #ifdef HAVE_HIGH_DPI_SCALE_FACTOR_ROUNDING_POLICY
-        if (kritarc.value("EnableHiDPIFractionalScaling", true).toBool()) {
+        if (kritarc.value("EnableHiDPIFractionalScaling", false).toBool()) {
             QGuiApplication::setHighDpiScaleFactorRoundingPolicy(Qt::HighDpiScaleFactorRoundingPolicy::PassThrough);
         }
 #endif
