@@ -829,6 +829,10 @@ void KisImage::purgeUnusedData(bool isCancellable)
                    deviceList << node->getLodCapableDevices();
                  });
 
+            /// make sure we deduplicate the list to avoid
+            /// concurrent write access to the devices
+            KritaUtils::makeContainerUnique(deviceList);
+
             Q_FOREACH (KisPaintDeviceSP device, deviceList) {
                 if (!device) continue;
 
@@ -1583,6 +1587,15 @@ QImage KisImage::convertToQImage(const QSize& scaledImageSize, const KoColorProf
     double scaleX = qreal(scaledImageSize.width()) / width();
     double scaleY = qreal(scaledImageSize.height()) / height();
 
+
+    if (scaleX < 1.0/256 || scaleY < 1.0/256) {
+        // quick checking if we're not trying to scale too much
+        // convertToQImage uses KisFixedPoint values, which means that the scale cannot be smaller than 1/2^8
+        // BUG:432182
+        // FIXME: would be best to extend KisFixedPoint instead
+        return convertToQImage(size(), profile).scaled(scaledImageSize, Qt::KeepAspectRatio, Qt::SmoothTransformation);
+    }
+
     QPointer<KoUpdater> updater = new KoDummyUpdater();
 
     KisTransformWorker worker(dev, scaleX, scaleY, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, updater, KisFilterStrategyRegistry::instance()->value("Bicubic"));
@@ -1951,6 +1964,11 @@ void KisImage::requestUndoDuringStroke()
     emit sigUndoDuringStrokeRequested();
 }
 
+void KisImage::requestRedoDuringStroke()
+{
+    emit sigRedoDuringStrokeRequested();
+}
+
 void KisImage::requestStrokeCancellation()
 {
     if (!m_d->tryCancelCurrentStrokeAsync()) {
