@@ -52,6 +52,7 @@
 #include "commands/kis_node_property_list_command.h"
 #include "commands/kis_node_compositeop_command.h"
 #include <KisDelayedUpdateNodeInterface.h>
+#include <KisCroppedOriginalLayerInterface.h>
 #include "krita_utils.h"
 #include "kis_image_signal_router.h"
 
@@ -124,8 +125,8 @@ namespace KisLayerUtils {
 
         KisNodeList allSrcNodes() override {
             KisNodeList mergedNodes;
-            mergedNodes << currLayer;
             mergedNodes << prevLayer;
+            mergedNodes << currLayer;
             return mergedNodes;
         }
     };
@@ -300,38 +301,10 @@ namespace KisLayerUtils {
                 m_info->image->bounds() : QRect();
 
             foreach (KisNodeSP node, m_info->allSrcNodes()) {
-                refreshHiddenAreaAsync(node, preparedRect);
+                refreshHiddenAreaAsync(m_info->image, node, preparedRect);
             }
         }
 
-    private:
-        QRect realNodeExactBounds(KisNodeSP rootNode, QRect currentRect = QRect()) {
-            KisNodeSP node = rootNode->firstChild();
-
-            while(node) {
-                currentRect |= realNodeExactBounds(node, currentRect);
-                node = node->nextSibling();
-            }
-
-            // TODO: it would be better to count up changeRect inside
-            // node's extent() method
-            currentRect |= rootNode->projectionPlane()->changeRect(rootNode->exactBounds());
-
-            return currentRect;
-        }
-
-        void refreshHiddenAreaAsync(KisNodeSP rootNode, const QRect &preparedArea) {
-            QRect realNodeRect = realNodeExactBounds(rootNode);
-            if (!preparedArea.contains(realNodeRect)) {
-
-                QRegion dirtyRegion = realNodeRect;
-                dirtyRegion -= preparedArea;
-
-                foreach(const QRect &rc, dirtyRegion.rects()) {
-                    m_info->image->refreshGraphAsync(rootNode, rc, realNodeRect);
-                }
-            }
-        }
     private:
         MergeDownInfoBaseSP m_info;
     };
@@ -408,6 +381,7 @@ namespace KisLayerUtils {
             }
 
             m_info->dstNode->setUseInTimeline(m_info->useInTimeline);
+            m_info->dstNode->setColorLabelIndex(m_info->allSrcNodes().first()->colorLabelIndex());
 
             KisPaintLayer *dstPaintLayer = qobject_cast<KisPaintLayer*>(m_info->dstNode.data());
             if (dstPaintLayer) {
@@ -487,6 +461,8 @@ namespace KisLayerUtils {
             m_info->nodesCompositingVaries = compositionVaries;
 
             m_info->dstNode->setUseInTimeline(m_info->useInTimeline);
+            m_info->dstNode->setColorLabelIndex(m_info->allSrcNodes().first()->colorLabelIndex());
+
             dstPaintLayer->setOnionSkinEnabled(m_info->enableOnionSkins);
         }
 
@@ -716,6 +692,52 @@ namespace KisLayerUtils {
     };
 
 
+    void splitNonRemovableNodes(KisNodeList &nodesToRemove, KisNodeList &_nodesToHide)
+    {
+        QSet<KisNodeSP> nodesToHide;
+        QSet<KisNodeSP> extraNodesToRemove;
+
+        for (auto it = nodesToRemove.begin(); it != nodesToRemove.end(); ++it) {
+            KisNodeSP root = *it;
+            KIS_SAFE_ASSERT_RECOVER_NOOP(root->visible());
+
+            if (!root->isEditable(false)) {
+                nodesToHide.insert(root);
+            } else {
+                bool rootNeedsCarefulRemoval = false;
+
+                recursiveApplyNodes(root,
+                                    [root, &nodesToHide, &rootNeedsCarefulRemoval] (KisNodeSP node) {
+                                        if (!node->isEditable(false)) {
+                                            while (node != root) {
+                                                nodesToHide.insert(node);
+                                                node = node->parent();
+                                                KIS_SAFE_ASSERT_RECOVER_BREAK(node);
+                                            }
+                                            nodesToHide.insert(root);
+                                            rootNeedsCarefulRemoval = true;
+                                        }
+                                    });
+
+                if (rootNeedsCarefulRemoval) {
+                    recursiveApplyNodes(root,
+                                        [&extraNodesToRemove] (KisNodeSP node) {
+                                            extraNodesToRemove.insert(node);
+                                        });
+                }
+            }
+        }
+
+        nodesToRemove += extraNodesToRemove.toList();
+
+        KritaUtils::filterContainer<KisNodeList>(nodesToRemove,
+                                                 [nodesToHide](KisNodeSP node) {
+                                                     return !nodesToHide.contains(node);
+                                                 });
+
+        _nodesToHide = nodesToHide.toList();
+    }
+
     struct CleanUpNodes : private RemoveNodeHelper, public KisCommandUtils::AggregateCommand {
         CleanUpNodes(MergeDownInfoBaseSP info, KisNodeSP putAfter)
             : m_info(info), m_putAfter(putAfter) {}
@@ -778,6 +800,18 @@ namespace KisLayerUtils {
                 KisNodeSP oldRoot = m_info->image->root();
                 KisNodeSP newRoot(new KisGroupLayer(m_info->image, "root", OPACITY_OPAQUE_U8));
 
+                // copy all fake nodes into the new image
+                KisLayerUtils::recursiveApplyNodes(oldRoot, [this, oldRoot, newRoot] (KisNodeSP node) {
+                    if (node->isFakeNode() && node->parent() == oldRoot) {
+                        addCommand(new KisImageLayerAddCommand(m_info->image,
+                                                               node->clone(),
+                                                               newRoot,
+                                                               KisNodeSP(),
+                                                               false, false));
+
+                    }
+                });
+
                 addCommand(new KisImageLayerAddCommand(m_info->image,
                                                        m_info->dstNode,
                                                        newRoot,
@@ -804,16 +838,14 @@ namespace KisLayerUtils {
                 }
 
                 KisNodeList safeNodesToDelete = m_info->allSrcNodes();
-                for (KisNodeList::iterator it = safeNodesToDelete.begin(); it != safeNodesToDelete.end(); ++it) {
-                    KisNodeSP node = *it;
-                    if (node->userLocked() && node->visible()) {
-                        addCommand(new KisImageChangeVisibilityCommand(false, node));
-                    }
+                KisNodeList safeNodesToHide;
+
+                splitNonRemovableNodes(safeNodesToDelete, safeNodesToHide);
+
+                Q_FOREACH(KisNodeSP node, safeNodesToHide) {
+                    addCommand(new KisImageChangeVisibilityCommand(false, node));
                 }
 
-                KritaUtils::filterContainer<KisNodeList>(safeNodesToDelete, [](KisNodeSP node) {
-                  return !node->userLocked();
-                });
                 safeRemoveMultipleNodes(safeNodesToDelete, m_info->image);
             }
 
@@ -915,6 +947,8 @@ namespace KisLayerUtils {
     }
 
     QSet<int> fetchLayerFramesRecursive(KisNodeSP rootNode) {
+        if (!rootNode->visible()) return QSet<int>();
+
         QSet<int> frames = fetchLayerFrames(rootNode);
 
         KisNodeSP node = rootNode->firstChild();
@@ -948,6 +982,9 @@ namespace KisLayerUtils {
         }
     }
 
+    /**
+     * \see a comment in mergeMultipleLayersImpl()
+     */
     void mergeDown(KisImageSP image, KisLayerSP layer, const KisMetaData::MergeStrategy* strategy)
     {
         if (!layer->prevSibling()) return;
@@ -1164,6 +1201,30 @@ namespace KisLayerUtils {
         return result;
     }
 
+    KisNodeList sortAndFilterAnyMergableNodesSafe(const KisNodeList &nodes, KisImageSP image) {
+        KisNodeList filteredNodes = nodes;
+        KisNodeList sortedNodes;
+
+        KisLayerUtils::filterMergableNodes(filteredNodes, true);
+
+        bool haveExternalNodes = false;
+        Q_FOREACH (KisNodeSP node, nodes) {
+            if (node->graphListener() != image->root()->graphListener()) {
+                haveExternalNodes = true;
+                break;
+            }
+        }
+
+        if (!haveExternalNodes) {
+            KisLayerUtils::sortMergableNodes(image->root(), filteredNodes, sortedNodes);
+        } else {
+            sortedNodes = filteredNodes;
+        }
+
+        return sortedNodes;
+    }
+
+
     void addCopyOfNameTag(KisNodeSP node)
     {
         const QString prefix = i18n("Copy of");
@@ -1248,6 +1309,31 @@ namespace KisLayerUtils {
         applicator.end();
     }
 
+    /**
+     * There might be two approaches for merging multiple layers:
+     *
+     * 1) Consider the selected nodes as a distinct "group" and merge them
+     *    as if they were isolated from the rest of the image. The key point
+     *    of this approach is that the look of the image will change, when
+     *    merging "weird" layers, like adjustment layers or layers with
+     *    non-normal blending mode.
+     *
+     * 2) Merge layers in a way to keep the look of the image as unchanged as
+     *    possible. With this approach one uses a few heuristics:
+     *
+     *       * when merging multiple layers with non-normal (but equal) blending
+     *         mode, first merge these layers together using Normal blending mode,
+     *         then set blending mode of the result to the original blending mode
+     *
+     *       * when merging multiple layers with different blending modes or
+     *         layer styles, they are first rasterized, and then laid over each
+     *         other with their own composite op. The blending mode of the final
+     *         layer is set to Normal, so the user could clearly see that he should
+     *         choose the correct blending mode.
+     *
+     * Krita uses the second approach: after merge operation, the image should look
+     * as if nothing has happened (if it is technically possible).
+     */
     void mergeMultipleLayersImpl(KisImageSP image, KisNodeList mergedNodes, KisNodeSP putAfter,
                                            bool flattenSingleLayer, const KUndo2MagicString &actionName,
                                            bool cleanupNodes = true, const QString layerName = QString())
@@ -1270,20 +1356,48 @@ namespace KisLayerUtils {
         emitSignals << ModifiedSignal;
         emitSignals << ComplexNodeReselectionSignal(KisNodeSP(), KisNodeList(), KisNodeSP(), mergedNodes);
 
-        KisProcessingApplicator applicator(image, 0,
-                                           KisProcessingApplicator::NONE,
-                                           emitSignals,
-                                           actionName);
 
 
         KisNodeList originalNodes = mergedNodes;
         KisNodeList invisibleNodes;
         mergedNodes = filterInvisibleNodes(originalNodes, &invisibleNodes, &putAfter);
 
-        if (!invisibleNodes.isEmpty()) {
+        if (mergedNodes.isEmpty()) return;
+
+
+        // make sure we don't add the new layer into a locked group
+        KIS_SAFE_ASSERT_RECOVER_RETURN(putAfter->parent());
+        while (putAfter->parent() && !putAfter->parent()->isEditable()) {
+            putAfter = putAfter->parent();
+        }
+
+        /**
+         * We have reached the root of the layer hierarchy and didn't manage
+         * to find a node that was editable enough for putting our merged
+         * result into it. That whouldn't happen in normal circumstances,
+         * unless the user chose to make the root layer visible and lock
+         * it manually.
+         */
+        if (!putAfter->parent()) {
+            return;
+        }
+
+        KisProcessingApplicator applicator(image, 0,
+                                           KisProcessingApplicator::NONE,
+                                           emitSignals,
+                                           actionName);
+
+
+        if (!invisibleNodes.isEmpty() && cleanupNodes) {
+
             /* If the putAfter node is invisible,
              * we should instead pick one of the nodes
-             * to be merged to avoid a null putAfter.
+             * to be merged to avoid a null putAfter
+             * after we remove all invisible layers from
+             * the image.
+             * (The assumption is that putAfter is among
+             * the layers to merge, so if it's invisible,
+             * it's going to be removed)
              */
             if (!putAfter->visible()){
                 putAfter = mergedNodes.first();
@@ -1312,7 +1426,7 @@ namespace KisLayerUtils {
             applicator.applyCommand(new DisableExtraCompositing(info));
             applicator.applyCommand(new KUndo2Command(), KisStrokeJobData::BARRIER);
 
-            if (info->frames.size() > 0) {
+            if (!info->frames.isEmpty()) {
                 foreach (int frame, info->frames) {
                     applicator.applyCommand(new SwitchFrameCommand(info->image, frame, false, info->storage));
 
@@ -1386,7 +1500,7 @@ namespace KisLayerUtils {
                     mask->selection()->pixelSelection(), SELECTION_ADD);
             }
 
-            KisSelectionMaskSP mergedMask = new KisSelectionMask(m_info->image);
+            KisSelectionMaskSP mergedMask = new KisSelectionMask(m_info->image, i18n("Selection Mask"));
             mergedMask->initSelection(parentLayer);
             mergedMask->setSelection(selection);
 
@@ -1540,6 +1654,18 @@ namespace KisLayerUtils {
             });
     }
 
+    void forceAllHiddenOriginalsUpdate(KisNodeSP root)
+    {
+        KisLayerUtils::recursiveApplyNodes(root,
+        [] (KisNodeSP node) {
+            KisCroppedOriginalLayerInterface *croppedUpdate =
+                    dynamic_cast<KisCroppedOriginalLayerInterface*>(node.data());
+            if (croppedUpdate) {
+                croppedUpdate->forceUpdateHiddenAreaOnOriginal();
+            }
+        });
+    }
+
     KisImageSP findImageByHierarchy(KisNodeSP node)
     {
         while (node) {
@@ -1554,4 +1680,88 @@ namespace KisLayerUtils {
         return 0;
     }
 
+    namespace Private {
+    QRect realNodeChangeRect(KisNodeSP rootNode, QRect currentRect = QRect()) {
+        KisNodeSP node = rootNode->firstChild();
+
+        while(node) {
+            currentRect |= realNodeChangeRect(node, currentRect);
+            node = node->nextSibling();
+        }
+
+        if (!rootNode->isFakeNode()) {
+            // TODO: it would be better to count up changeRect inside
+            // node's extent() method
+            currentRect |= rootNode->projectionPlane()->changeRect(rootNode->exactBounds());
+        }
+
+        return currentRect;
+    }
+    }
+
+    void refreshHiddenAreaAsync(KisImageSP image, KisNodeSP rootNode, const QRect &preparedArea) {
+        QRect realNodeRect = Private::realNodeChangeRect(rootNode);
+        if (!preparedArea.contains(realNodeRect)) {
+
+            QRegion dirtyRegion = realNodeRect;
+            dirtyRegion -= preparedArea;
+
+            auto rc = dirtyRegion.begin();
+            while (rc != dirtyRegion.end()) {
+                image->refreshGraphAsync(rootNode, *rc, realNodeRect);
+                rc++;
+            }
+        }
+    }
+
+    QRect recursiveTightNodeVisibleBounds(KisNodeSP rootNode)
+    {
+        QRect exactBounds;
+        recursiveApplyNodes(rootNode, [&exactBounds] (KisNodeSP node) {
+            exactBounds |= node->projectionPlane()->tightUserVisibleBounds();
+        });
+        return exactBounds;
+    }
+
+    KisNodeSP findRoot(KisNodeSP node)
+    {
+        if (!node) return node;
+
+        while (node->parent()) {
+            node = node->parent();
+        }
+        return node;
+    }
+
+    bool canChangeImageProfileInvisibly(KisImageSP image)
+    {
+        int numLayers = 0;
+        bool hasNonNormalLayers = false;
+        bool hasTransparentLayer = false;
+
+
+        recursiveApplyNodes(image->root(),
+            [&numLayers, &hasNonNormalLayers, &hasTransparentLayer, image] (KisNodeSP node) {
+                if (!node->inherits("KisLayer")) return;
+
+                numLayers++;
+
+                if (node->exactBounds().isEmpty()) return;
+
+                // this is only an approximation! it is not exact!
+                if (!hasTransparentLayer &&
+                    node->exactBounds() != image->bounds()) {
+
+                    hasTransparentLayer = true;
+                }
+
+                if (!hasNonNormalLayers &&
+                    node->compositeOpId() != COMPOSITE_OVER) {
+
+                    hasNonNormalLayers = true;
+                }
+            });
+
+        return numLayers == 1 || (!hasNonNormalLayers && !hasTransparentLayer);
+    }
 }
