@@ -846,21 +846,21 @@ void LCHToLab(const qreal L, const qreal C, const qreal H, qreal *l, qreal *a, q
 
 void XYZToxyY(const qreal X, const qreal Y, const qreal Z, qreal *x, qreal *y, qreal *yY)
 {
-    qBound(0.0,X,1.0);
-    qBound(0.0,Y,1.0);
-    qBound(0.0,Z,1.0);
-    *x=X/(X+Y+Z);
-    *y=Y/(X+Y+Z);
-    *yY=Y;
+    qreal Xb = qBound(0.0,X,1.0);
+    qreal Yb = qBound(0.0,Y,1.0);
+    qreal Zb = qBound(0.0,Z,1.0);
+    *x=Xb/(Xb+Yb+Zb);
+    *y=Yb/(Xb+Yb+Zb);
+    *yY=Yb;
 }  
 void xyYToXYZ(const qreal x, const qreal y, const qreal yY, qreal *X, qreal *Y, qreal *Z)
 {
-    qBound(0.0,x,1.0);
-    qBound(0.0,y,1.0);
-    qBound(0.0,yY,1.0);
-    *X=(x*yY)/y;
-    *Z=((1.0-x-y)/yY)/y;
-    *Y=yY;
+    qreal xb = qBound(0.0,x,1.0);
+    qreal yb = qBound(0.0,y,1.0);
+    qreal yYb = qBound(0.0,yY,1.0);
+    *X=(xb*yYb)/yb;
+    *Z=((1.0-xb-yb)*yYb)/yb;
+    *Y=yYb;
 }
 
 void CMYToCMYK(qreal *c, qreal *m, qreal *y, qreal *k)
