@@ -1,6 +1,6 @@
 /*  This file is part of the KDE project
    Copyright (c) 2005 Boudewijn Rempt <boud@valdyas.org>
-   Copyright (c) 2016 L. E. Segovia <leo.segovia@siggraph.org>
+   Copyright (c) 2016 L. E. Segovia <amy@amyspark.me>
 
 
     This library is free software; you can redistribute it and/or
@@ -259,7 +259,7 @@ void KoColorSet::setPaletteType(PaletteType paletteType)
 
 quint32 KoColorSet::colorCount() const
 {
-    int colorCount = d->groups[GLOBAL_GROUP_NAME].colorCount();
+    int colorCount = 0;
     for (KisSwatchGroup &g : d->groups.values()) {
         colorCount += g.colorCount();
     }
@@ -290,20 +290,16 @@ void KoColorSet::clear()
 
 KisSwatch KoColorSet::getColorGlobal(quint32 x, quint32 y) const
 {
-    int yInGroup = y;
-    QString nameGroupFoundIn;
-    for (const QString &groupName : d->groupNames) {
-        if (yInGroup < d->groups[groupName].rowCount()) {
-            nameGroupFoundIn = groupName;
-            break;
-        } else {
-            yInGroup -= d->groups[groupName].rowCount();
+    for (const QString &groupName : getGroupNames()) {
+        if (d->groups.contains(groupName)) {
+            if ((int)y < d->groups[groupName].rowCount()) {
+                return d->groups[groupName].getEntry(x, y);
+            } else {
+                y -= d->groups[groupName].rowCount();
+            }
         }
     }
-    const KisSwatchGroup &groupFoundIn = nameGroupFoundIn == GLOBAL_GROUP_NAME
-            ? d->global() : d->groups[nameGroupFoundIn];
-    Q_ASSERT(groupFoundIn.checkEntry(x, yInGroup));
-    return groupFoundIn.getEntry(x, yInGroup);
+    return KisSwatch();
 }
 
 KisSwatch KoColorSet::getColorGroup(quint32 x, quint32 y, QString groupName)
@@ -317,7 +313,7 @@ KisSwatch KoColorSet::getColorGroup(quint32 x, quint32 y, QString groupName)
     return e;
 }
 
-QStringList KoColorSet::getGroupNames()
+QStringList KoColorSet::getGroupNames() const
 {
     if (d->groupNames.size() != d->groups.size()) {
         warnPigment << "mismatch between groups and the groupnames list.";
@@ -345,8 +341,8 @@ bool KoColorSet::changeGroupName(const QString &oldGroupName, const QString &new
 
 void KoColorSet::setColumnCount(int columns)
 {
-    d->groups[GLOBAL_GROUP_NAME].setColumnCount(columns);
-    for (KisSwatchGroup &g : d->groups.values()) {
+    for (auto it = d->groups.begin(); it != d->groups.end(); ++it) {
+        KisSwatchGroup &g = *it;
         g.setColumnCount(columns);
     }
 }
@@ -368,7 +364,7 @@ void KoColorSet::setComment(QString comment)
 
 bool KoColorSet::addGroup(const QString &groupName)
 {
-    if (d->groups.contains(groupName) || d->groupNames.contains(groupName)) {
+    if (d->groups.contains(groupName) || getGroupNames().contains(groupName)) {
         return false;
     }
     d->groupNames.append(groupName);
@@ -379,7 +375,7 @@ bool KoColorSet::addGroup(const QString &groupName)
 
 bool KoColorSet::moveGroup(const QString &groupName, const QString &groupNameInsertBefore)
 {
-    if (d->groupNames.contains(groupName)==false || d->groupNames.contains(groupNameInsertBefore)==false) {
+    if (!d->groupNames.contains(groupName) || d->groupNames.contains(groupNameInsertBefore)==false) {
         return false;
     }
     if (groupNameInsertBefore != GLOBAL_GROUP_NAME && groupName != GLOBAL_GROUP_NAME) {
@@ -424,7 +420,7 @@ QString KoColorSet::defaultFileExtension() const
 int KoColorSet::rowCount() const
 {
     int res = 0;
-    for (const QString &name : d->groupNames) {
+    for (const QString &name : getGroupNames()) {
         res += d->groups[name].rowCount();
     }
     return res;
@@ -495,8 +491,6 @@ KisSwatchGroup::SwatchInfo KoColorSet::getClosestColorInfo(KoColor compare, bool
 
 KoColorSet::Private::Private(KoColorSet *a_colorSet)
     : colorSet(a_colorSet)
-    , isGlobal(true)
-    , isEditable(false)
 {
     groups[KoColorSet::GLOBAL_GROUP_NAME] = KisSwatchGroup();
     groupNames.append(KoColorSet::GLOBAL_GROUP_NAME);
@@ -832,7 +826,7 @@ bool KoColorSet::Private::loadGpl()
                 a.pop_front();
             }
             QString name = a.join(" ");
-            e.setName(name.isEmpty() ? i18n("Untitled") : name);
+            e.setName(name.isEmpty() || name == "Untitled" ? i18n("Untitled") : name);
 
             global().addEntry(e);
         }
@@ -862,7 +856,7 @@ bool KoColorSet::Private::loadAct()
 
 bool KoColorSet::Private::loadRiff()
 {
-    // http://worms2d.info/Palette_file
+    // https://worms2d.info/Palette_file
     QFileInfo info(colorSet->filename());
     colorSet->setName(info.completeBaseName());
     KisSwatch e;
