@@ -176,12 +176,8 @@ public:
     }
 
     void setIndex(int idx) override {
-        KisImageWSP image = this->image();
-        image->requestStrokeCancellation();
-        if(image->tryBarrierLock()) {
-            KUndo2Stack::setIndex(idx);
-            image->unlock();
-        }
+        m_postponedJobs.append(PostponedJob(PostponedJob::SetIndex, idx));
+        processPostponedJobs();
     }
 
     void notifySetIndexChangedOneCommand() override {
@@ -199,6 +195,33 @@ public:
     }
 
     void undo() override {
+        m_postponedJobs.append(PostponedJob(PostponedJob::Undo, 0));
+        processPostponedJobs();
+    }
+
+
+    void redo() override {
+        m_postponedJobs.append(PostponedJob(PostponedJob::Redo, 0));
+        processPostponedJobs();
+    }
+
+private:
+    KisImageWSP image() {
+        KisImageWSP currentImage = m_doc->image();
+        Q_ASSERT(currentImage);
+        return currentImage;
+    }
+
+    void setIndexImpl(int idx) {
+        KisImageWSP image = this->image();
+        image->requestStrokeCancellation();
+        if(image->tryBarrierLock()) {
+            KUndo2Stack::setIndex(idx);
+            image->unlock();
+        }
+    }
+
+    void undoImpl() {
         KisImageWSP image = this->image();
         image->requestUndoDuringStroke();
 
@@ -212,22 +235,69 @@ public:
         }
     }
 
-    void redo() override {
+    void redoImpl() {
         KisImageWSP image = this->image();
+        image->requestRedoDuringStroke();
+
         if(image->tryBarrierLock()) {
             KUndo2Stack::redo();
             image->unlock();
         }
     }
 
-private:
-    KisImageWSP image() {
-        KisImageWSP currentImage = m_doc->image();
-        Q_ASSERT(currentImage);
-        return currentImage;
+    void processPostponedJobs() {
+        /**
+         * Some undo commands may call QApplication::processEvents(),
+         * see notifySetIndexChangedOneCommand(). That may cause
+         * recursive calls to the undo stack methods when used from
+         * the Undo History docker. Here we try to handle that gracefully
+         * by accumulating all the requests and executing them at the
+         * topmost level of recursion.
+         */
+        if (m_recursionCounter > 0) return;
+
+        m_recursionCounter++;
+
+        while (!m_postponedJobs.isEmpty()) {
+            PostponedJob job = m_postponedJobs.dequeue();
+            switch (job.type) {
+            case PostponedJob::SetIndex:
+                setIndexImpl(job.index);
+                break;
+            case PostponedJob::Redo:
+                redoImpl();
+                break;
+            case PostponedJob::Undo:
+                undoImpl();
+                break;
+            }
+        }
+
+        m_recursionCounter--;
     }
 
 private:
+    int m_recursionCounter = 0;
+
+    struct PostponedJob {
+        enum Type {
+            Undo = 0,
+            Redo,
+            SetIndex
+        };
+
+        PostponedJob() {}
+        PostponedJob(Type _type, int _index)
+         : type(_type),
+           index(_index)
+        {
+        }
+
+        Type type = Undo;
+        int index = 0;
+    };
+    QQueue<PostponedJob> m_postponedJobs;
+
     KisDocument *m_doc;
 };
 
@@ -337,6 +407,7 @@ public:
 
     StdLockableWrapper<QMutex> savingLock;
 
+    bool imageModifiedWithoutUndo = false;
     bool modifiedWhileSaving = false;
     QScopedPointer<KisDocument> backgroundSaveDocument;
     QPointer<KoUpdater> savingUpdater;
@@ -440,6 +511,7 @@ void KisDocument::Private::copyFromImpl(const Private &rhs, KisDocument *q, KisD
         assistants = KisPaintingAssistant::cloneAssistantList(rhs.assistants);
         gridConfig = rhs.gridConfig;
     }
+    imageModifiedWithoutUndo = rhs.imageModifiedWithoutUndo;
     m_bAutoDetectedMime = rhs.m_bAutoDetectedMime;
     m_url = rhs.m_url;
     m_file = rhs.m_file;
@@ -810,6 +882,7 @@ void KisDocument::slotCompleteSavingDocument(const KritaUtils::ExportFileJob &jo
                 if (d->undoStack->isClean()) {
                     setModified(false);
                 } else {
+                    d->imageModifiedWithoutUndo = false;
                     d->undoStack->setClean();
                 }
             }
@@ -1689,6 +1762,10 @@ void KisDocument::setModified(bool mod)
     d->modifiedAfterAutosave = mod;
     d->modifiedWhileSaving = mod;
 
+    if (!mod) {
+        d->imageModifiedWithoutUndo = mod;
+    }
+
     if (mod == isModified())
         return;
 
@@ -1883,7 +1960,7 @@ void KisDocument::endMacro()
 
 void KisDocument::slotUndoStackCleanChanged(bool value)
 {
-    setModified(!value);
+    setModified(!value || d->imageModifiedWithoutUndo);
 }
 
 void KisDocument::slotConfigChanged()
@@ -2106,6 +2183,7 @@ bool KisDocument::newImage(const QString& name,
     Q_CHECK_PTR(image);
 
     connect(image, SIGNAL(sigImageModified()), this, SLOT(setImageModified()), Qt::UniqueConnection);
+    connect(image, SIGNAL(sigImageModifiedWithoutUndo()), this, SLOT(setImageModifiedWithoutUndo()), Qt::UniqueConnection);
     image->setResolution(imageResolution, imageResolution);
 
     image->assignImageProfile(cs->profile());
@@ -2294,6 +2372,7 @@ void KisDocument::setCurrentImage(KisImageSP image, bool forceInitialUpdate)
     d->shapeController->setImage(image);
     setModified(false);
     connect(d->image, SIGNAL(sigImageModified()), this, SLOT(setImageModified()), Qt::UniqueConnection);
+    connect(d->image, SIGNAL(sigImageModifiedWithoutUndo()), this, SLOT(setImageModifiedWithoutUndo()), Qt::UniqueConnection);
     connect(d->image, SIGNAL(sigLayersChangedAsync()), this, SLOT(slotImageRootChanged()));
 
     if (forceInitialUpdate) {
@@ -2314,7 +2393,13 @@ void KisDocument::hackPreliminarySetImage(KisImageSP image)
 void KisDocument::setImageModified()
 {
     // we only set as modified if undo stack is not at clean state
-    setModified(!d->undoStack->isClean());
+    setModified(d->imageModifiedWithoutUndo || !d->undoStack->isClean());
+}
+
+void KisDocument::setImageModifiedWithoutUndo()
+{
+    d->imageModifiedWithoutUndo = true;
+    setImageModified();
 }
 
 
