@@ -257,6 +257,12 @@ KisView::~KisView()
         d->viewManager->mainWindow()->notifyChildViewDestroyed(this);
     }
 
+    /**
+     * KisCanvas2 maintains direct connections to the image, so we should
+     * disconnect it from the image before the destruction process starts
+     */
+    d->canvas.disconnectImage();
+
     KoToolManager::instance()->removeCanvasController(&d->canvasController);
     d->canvasController.setCanvas(0);
 
