@@ -44,6 +44,10 @@
 #include <QFileDialog>
 #include <QFormLayout>
 #include <QSettings>
+#include <QStyleFactory>
+#include <QScreen>
+#include <QFontComboBox>
+#include <QFont>
 
 #include <KisDocument.h>
 #include <KoColorProfile.h>
@@ -94,6 +98,39 @@
 #include "config-high-dpi-scale-factor-rounding-policy.h"
 #endif
 
+QString shortNameOfDisplay(QScreen* screen)
+{
+    // Depending on the display, all of those properties might be
+    // or might not be useful
+    // Example:
+    //	Screen: 0
+    //    Name: eDP-1
+    //    Manufacturer: BOE
+    //    Model:
+    //	Screen: 1
+    //    Name: DP-2
+    //    Manufacturer: Toshiba America Info Systems Inc
+    //    Model: TOSHIBA-TV-
+    // In the first case, model is empty, manufacturer is BOE
+    // The second case model is more useful than manufacturer because it's short
+    // and it gives basically the same amount of information
+    QString name = screen->name();
+    QString model = screen->model();
+    QString manufacturer = screen->manufacturer();
+    QString resolution = QString::number(screen->geometry().width()).append("x").append(QString::number(screen->geometry().height()));
+
+    QString shortName = name + " ";
+    if (!model.isEmpty()) {
+        shortName += model;
+    } else {
+        shortName += manufacturer;
+    }
+    shortName = shortName.left(15);
+    shortName = shortName.append(" ").append(resolution);
+    return shortName;
+}
+
+
 struct BackupSuffixValidator : public QValidator {
     BackupSuffixValidator(QObject *parent)
         : QValidator(parent)
@@ -155,6 +192,28 @@ GeneralTab::GeneralTab(QWidget *_parent, const char *_name)
     //
     // Window Tab
     //
+    chkUseCustomFont->setChecked(cfg.readEntry<bool>("use_custom_system_font", false));
+    cmbCustomFont->setEnabled(cfg.readEntry<bool>("use_custom_system_font", false));
+    cmbCustomFont->findChild <QComboBox*>("stylesComboBox")->setVisible(false);
+    intFontSize->setEnabled(cmbCustomFont->isEnabled());
+
+    QString fontName = cfg.readEntry<QString>("custom_system_font", "");
+    if (fontName.isEmpty()) {
+        cmbCustomFont->setCurrentFont(qApp->font());
+
+    }
+    else {
+        int pointSize = qApp->font().pointSize();
+        cmbCustomFont->setCurrentFont(QFont(fontName, pointSize));
+    }
+    int fontSize = cfg.readEntry<int>("custom_font_size", -1);
+    if (fontSize < 0) {
+        intFontSize->setValue(qApp->font().pointSize());
+    }
+    else {
+        intFontSize->setValue(fontSize);
+    }
+
     m_cmbMDIType->setCurrentIndex(cfg.readEntry<int>("mdi_viewmode", (int)QMdiArea::TabbedView));
 
     m_backgroundimage->setText(cfg.getMDIBackgroundImage());
@@ -173,7 +232,7 @@ GeneralTab::GeneralTab(QWidget *_parent, const char *_name)
     QSettings kritarc(configPath + QStringLiteral("/kritadisplayrc"), QSettings::IniFormat);
     m_chkHiDPI->setChecked(kritarc.value("EnableHiDPI", true).toBool());
 #ifdef HAVE_HIGH_DPI_SCALE_FACTOR_ROUNDING_POLICY
-    m_chkHiDPIFractionalScaling->setChecked(kritarc.value("EnableHiDPIFractionalScaling", true).toBool());
+    m_chkHiDPIFractionalScaling->setChecked(kritarc.value("EnableHiDPIFractionalScaling", false).toBool());
 #else
     m_chkHiDPIFractionalScaling->setVisible(false);
 #endif
@@ -283,6 +342,11 @@ void GeneralTab::setDefault()
     m_chkNativeFileDialog->setChecked(false);
     intMaxBrushSize->setValue(1000);
 
+    chkUseCustomFont->setChecked(false);
+    cmbCustomFont->setCurrentFont(qApp->font());
+    intFontSize->setValue(qApp->font().pointSize());
+
+        
     m_cmbMDIType->setCurrentIndex((int)QMdiArea::TabbedView);
     m_chkRubberBand->setChecked(cfg.useOpenGL(true));
     m_favoritePresetsSpinBox->setValue(cfg.favoritePresets(true));
@@ -535,7 +599,8 @@ ColorSettingsTab::ColorSettingsTab(QWidget *parent, const char *name)
 
     QFormLayout *monitorProfileGrid = new QFormLayout(m_page->monitorprofileholder);
     for(int i = 0; i < QGuiApplication::screens().count(); ++i) {
-        QLabel *lbl = new QLabel(i18nc("The number of the screen", "Screen %1:", i + 1));
+        QScreen* screen = QGuiApplication::screens()[i];
+        QLabel *lbl = new QLabel(i18nc("The number of the screen (ordinal) and shortened 'name' of the screen (model + resolution)", "Screen %1 (%2):", i + 1, shortNameOfDisplay(screen)));
         m_monitorProfileLabels << lbl;
         KisSqueezedComboBox *cmb = new KisSqueezedComboBox();
         cmb->setSizePolicy(QSizePolicy::MinimumExpanding, QSizePolicy::Fixed);
@@ -642,8 +707,9 @@ void ColorSettingsTab::toggleAllowMonitorProfileSelection(bool useSystemProfile)
             for(int i = 0; i < QApplication::screens().count(); ++i) {
                 m_monitorProfileWidgets[i]->clear();
                 QString monitorForScreen = cfg.monitorForScreen(i, devices[i]);
+                QScreen* screen = QGuiApplication::screens()[i];
                 Q_FOREACH (const QString &device, devices) {
-                    m_monitorProfileLabels[i]->setText(i18nc("The display/screen we got from Qt", "Screen %1:", i + 1));
+                    m_monitorProfileLabels[i]->setText(i18nc("The number of the screen (ordinal) and shortened 'name' of the screen (model + resolution)", "Screen %1 (%2):", i + 1, shortNameOfDisplay(screen)));
                     m_monitorProfileWidgets[i]->addSqueezedItem(KisColorManager::instance()->deviceName(device), device);
                     if (devices[i] == monitorForScreen) {
                         m_monitorProfileWidgets[i]->setCurrentIndex(i);
@@ -719,7 +785,8 @@ void ColorSettingsTab::refillMonitorProfiles(const KoID & colorSpaceId)
     }
 
     for (int i = 0; i < QApplication::screens().count(); ++i) {
-        m_monitorProfileLabels[i]->setText(i18nc("The number of the screen", "Screen %1:", i + 1));
+        QScreen* screen = QGuiApplication::screens()[i];
+        m_monitorProfileLabels[i]->setText(i18nc("The number of the screen (ordinal) and shortened 'name' of the screen (model + resolution)", "Screen %1 (%2):", i + 1, shortNameOfDisplay(screen)));
         m_monitorProfileWidgets[i]->setCurrent(KoColorSpaceRegistry::instance()->defaultProfileForColorSpace(colorSpaceId.id()));
     }
 }
@@ -1646,6 +1713,16 @@ bool KisDlgPreferences::editPreferences()
 
         cfg.writeEntry<int>("maximumBrushSize", m_general->intMaxBrushSize->value());
 
+        cfg.writeEntry<bool>("use_custom_system_font", m_general->chkUseCustomFont->isChecked());
+        if (m_general->chkUseCustomFont->isChecked()) {
+            cfg.writeEntry<QString>("custom_system_font", m_general->cmbCustomFont->currentFont().family());
+            cfg.writeEntry<int>("custom_font_size", m_general->intFontSize->value());
+        }
+        else {
+            cfg.writeEntry<QString>("custom_system_font", "");
+            cfg.writeEntry<int>("custom_font_size", -1);
+        }
+
         cfg.writeEntry<int>("mdi_viewmode", m_general->mdiMode());
         cfg.setMDIBackgroundColor(m_general->m_mdiColor->color().toXML());
         cfg.setMDIBackgroundImage(m_general->m_backgroundimage->text());
