@@ -49,10 +49,11 @@ public:
     void activate(int shortcut) override;
     void deactivate(int shortcut) override;
     void begin(int shortcut, QEvent *event) override;
-    void cursorMovedAbsolute(const QPointF &startPos, const QPointF &pos);
+    void cursorMovedAbsolute(const QPointF &startPos, const QPointF &pos) override;
     void inputEvent(QEvent* event) override;
 
     KisInputActionGroup inputActionGroup(int shortcut) const override;
+    bool supportsHiResInputEvents(int shortcut) const override;
 
 private:
     class Private;
