@@ -187,7 +187,7 @@ bool KisShortcutMatcher::supportsHiResInputEvents()
     return
         m_d->runningShortcut &&
         m_d->runningShortcut->action() &&
-        m_d->runningShortcut->action()->supportsHiResInputEvents();
+        m_d->runningShortcut->action()->supportsHiResInputEvents(m_d->runningShortcut->shortcutIndex());
 }
 
 bool KisShortcutMatcher::keyPressed(Qt::Key key)
@@ -408,6 +408,14 @@ bool KisShortcutMatcher::touchEndEvent( QTouchEvent* event )
     return false;
 }
 
+void KisShortcutMatcher::touchCancelEvent(const QPointF &localPos)
+{
+    m_d->usingTouch = false;
+    if (m_d->runningShortcut) {
+        forceEndRunningShortcut(localPos);
+    }
+}
+
 bool KisShortcutMatcher::nativeGestureBeginEvent(QNativeGestureEvent *event)
 {
     Q_UNUSED(event)
@@ -490,6 +498,26 @@ void KisShortcutMatcher::recoveryModifiersWithoutFocus(const QVector<Qt::Key> &k
     DEBUG_ACTION("recoverySyncModifiers");
 }
 
+bool KisShortcutMatcher::sanityCheckModifiersCorrectness(Qt::KeyboardModifiers modifiers) const
+{
+    auto checkKey = [this, modifiers] (Qt::Key key, Qt::KeyboardModifier modifier) {
+        return m_d->keys.contains(key) == bool(modifiers & modifier);
+    };
+
+    return checkKey(Qt::Key_Shift, Qt::ShiftModifier) &&
+        checkKey(Qt::Key_Control, Qt::ControlModifier) &&
+        checkKey(Qt::Key_Alt, Qt::AltModifier) &&
+        checkKey(Qt::Key_Meta, Qt::MetaModifier);
+
+}
+
+QVector<Qt::Key> KisShortcutMatcher::debugPressedKeys() const
+{
+    QVector<Qt::Key> keys;
+    std::copy(m_d->keys.begin(), m_d->keys.end(), std::back_inserter(keys));
+    return keys;
+}
+
 void KisShortcutMatcher::lostFocusEvent(const QPointF &localPos)
 {
     Private::RecursionNotifier notifier(this);
