@@ -69,7 +69,7 @@ struct TimelineFramesView::Private
           dragInProgress(false),
           dragWasSuccessful(false),
           modifiersCatcher(0),
-          selectionChangedCompressor(300, KisSignalCompressor::FIRST_INACTIVE)
+          selectionChangedCompressor(500, KisSignalCompressor::FIRST_INACTIVE)
     {}
 
     TimelineFramesView *q;
@@ -187,6 +187,9 @@ TimelineFramesView::TimelineFramesView(QWidget *parent)
     m_d->addLayersButton->setPopupMode(QToolButton::InstantPopup);
 
     m_d->layerEditingMenu = new QMenu(this);
+    m_d->layerEditingMenu->addSection(i18n("Edit Layers:"));
+    m_d->layerEditingMenu->addSeparator();
+
     m_d->layerEditingMenu->addAction(KisAnimationUtils::newLayerActionName, this, SLOT(slotAddNewLayer()));
     m_d->existingLayersMenu = m_d->layerEditingMenu->addMenu(KisAnimationUtils::addExistingLayerActionName);
     m_d->layerEditingMenu->addSeparator();
@@ -209,6 +212,8 @@ TimelineFramesView::TimelineFramesView(QWidget *parent)
     m_d->audioOptionsButton->setPopupMode(QToolButton::InstantPopup);
 
     m_d->audioOptionsMenu = new QMenu(this);
+    m_d->audioOptionsMenu->addSection(i18n("Edit Audio:"));
+    m_d->audioOptionsMenu->addSeparator();
 
 #ifndef HAVE_QT_MULTIMEDIA
     m_d->audioOptionsMenu->addSection(i18nc("@item:inmenu", "Audio playback is not supported in this build!"));
@@ -683,6 +688,7 @@ void TimelineFramesView::slotDataChanged(const QModelIndex &topLeft, const QMode
         return;
     }
 
+    if (selectionModel()->selectedIndexes().count() > 1) return;
 
     if (selectedColumn == -1) {
         selectedColumn = index.column();
@@ -692,6 +698,7 @@ void TimelineFramesView::slotDataChanged(const QModelIndex &topLeft, const QMode
         int row= index.isValid() ? index.row() : 0;
         selectionModel()->setCurrentIndex(m_d->model->index(row, selectedColumn), QItemSelectionModel::ClearAndSelect);
     }
+
 }
 
 void TimelineFramesView::slotHeaderDataChanged(Qt::Orientation orientation, int first, int last)
@@ -936,9 +943,10 @@ void TimelineFramesView::createFrameEditingMenuActions(QMenu *menu, bool addFram
     int minColumn = 0;
     int maxColumn = 0;
     calculateSelectionMetrics(minColumn, maxColumn, rows);
-
     bool selectionExists = minColumn != maxColumn;
 
+    menu->addSection(i18n("Edit Frames:"));
+    menu->addSeparator();
 
     if (selectionExists) {
         KisActionManager::safePopulateMenu(menu, "update_playback_range", m_d->actionMan);
@@ -955,6 +963,19 @@ void TimelineFramesView::createFrameEditingMenuActions(QMenu *menu, bool addFram
 
     menu->addSeparator();
 
+    {   // Tween submenu.
+        QMenu *frames = menu->addMenu(i18nc("@item:inmenu", "Tweening"));
+
+        KisActionManager::safePopulateMenu(frames, "insert_opacity_keyframe", m_d->actionMan);
+        KisActionManager::safePopulateMenu(frames, "remove_opacity_keyframe", m_d->actionMan);
+
+        // only allow to add an opacity keyframe if one doesn't exist
+        bool opacityKeyframeExists = model()->data(currentIndex(), TimelineFramesModel::SpecialKeyframeExists).toBool();
+        m_d->actionMan->actionByName("insert_opacity_keyframe")->setEnabled(!opacityKeyframeExists);
+        m_d->actionMan->actionByName("remove_opacity_keyframe")->setEnabled(opacityKeyframeExists);
+    }
+
+
     {   //Frames submenu.
         QMenu *frames = menu->addMenu(i18nc("@item:inmenu", "Keyframes"));
         KisActionManager::safePopulateMenu(frames, "insert_keyframe_left", m_d->actionMan);
@@ -1017,6 +1038,7 @@ void TimelineFramesView::mousePressEvent(QMouseEvent *event)
             model()->setData(index, true, TimelineFramesModel::ActiveFrameRole);
             setCurrentIndex(index);
 
+
             if (model()->data(index, TimelineFramesModel::FrameExistsRole).toBool() ||
                     model()->data(index, TimelineFramesModel::SpecialKeyframeExists).toBool()) {
 
@@ -1216,10 +1238,12 @@ void TimelineFramesView::slotUpdateFrameActions()
     enableAction("copy_frames_to_clipboard", true);
     enableAction("cut_frames_to_clipboard", hasEditableFrames);
 
+    enableAction("insert_opacity_keyframe", hasEditableFrames);
+    enableAction("remove_opacity_keyframe", hasEditableFrames);
+
     QClipboard *cp = QApplication::clipboard();
     const QMimeData *data = cp->mimeData();
 
-    enableAction("paste_frames_from_clipboard", data && data->hasFormat("application/x-krita-frame"));
 
     //TODO: update column actions!
 }
@@ -1443,12 +1467,47 @@ void TimelineFramesView::insertOrRemoveHoldFrames(int count, bool entireColumn)
 
         m_d->model->insertHoldFrames(indexes, count);
 
+        // Fan selection based on insertion or deletion.
+        // This should allow better UI/UX for insertion of keyframes or hold frames.
+        fanSelectedFrames(indexes, count);
+
         // bulk adding frames can add too many
         // trim timeline to clean up extra frames that might have been added
         slotUpdateInfiniteFramesCount();
     }
 }
 
+void TimelineFramesView::fanSelectedFrames(const QModelIndexList &selection, int count, bool ignoreKeyless) {
+    QMap<int, QList<int>> indexMap;
+
+    QList<QModelIndex> selectedIndices = selection;
+
+    foreach (const QModelIndex &index, selectedIndices) {
+        if (!indexMap.contains(index.row())) {
+            indexMap.insert(index.row(), QList<int>());
+        }
+
+        if (m_d->model->data(index, TimelineFramesModel::FrameExistsRole).value<bool>() || !ignoreKeyless) {
+            indexMap[index.row()] << index.column();
+        }
+    }
+
+    selectionModel()->clearSelection();
+    KisSignalsBlocker blockSig(selectionModel());
+    foreach (const int &layer, indexMap.keys()) {
+        QList<int>::const_iterator it;
+        int progressIndex = 0;
+
+        std::sort(indexMap[layer].begin(), indexMap[layer].end());
+        for (it = indexMap[layer].constBegin(); it != indexMap[layer].constEnd(); it++) {
+            const int offsetColumn = *it + (progressIndex * count);
+            selectionModel()->select(model()->index(layer, offsetColumn), QItemSelectionModel::Select);
+            progressIndex++;
+        }
+    }
+
+}
+
 void TimelineFramesView::insertOrRemoveMultipleHoldFrames(bool insertion, bool entireColumn)
 {
     bool ok = false;
