@@ -52,8 +52,12 @@ GridConfigWidget::GridConfigWidget(QWidget *parent) :
     ui->colorGuides->setAlphaChannelEnabled(true);
 
 
-    ui->angleLeftSpinbox->setSuffix(QChar(Qt::Key_degree));
-    ui->angleRightSpinbox->setSuffix(QChar(Qt::Key_degree));
+    ui->angleLeftAngleSelector->setRange(0, 89);
+    ui->angleRightAngleSelector->setRange(0, 89);
+    ui->angleLeftAngleSelector->setDecimals(0);
+    ui->angleRightAngleSelector->setDecimals(0);
+    ui->angleLeftAngleSelector->setFlipOptionsMode(KisAngleSelector::FlipOptionsMode_NoFlipOptions);
+    ui->angleRightAngleSelector->setFlipOptionsMode(KisAngleSelector::FlipOptionsMode_NoFlipOptions);
     ui->cellSpacingSpinbox->setSuffix(i18n(" px"));
 
 
@@ -91,8 +95,8 @@ GridConfigWidget::GridConfigWidget(QWidget *parent) :
     connect(ui->chkLockGuides, SIGNAL(stateChanged(int)), SLOT(slotGuidesGuiChanged()));
 
     connect(ui->intSubdivision, SIGNAL(valueChanged(int)), SLOT(slotGridGuiChanged()));
-    connect(ui->angleLeftSpinbox, SIGNAL(valueChanged(int)), SLOT(slotGridGuiChanged()));
-    connect(ui->angleRightSpinbox, SIGNAL(valueChanged(int)), SLOT(slotGridGuiChanged()));
+    connect(ui->angleLeftAngleSelector, SIGNAL(angleChanged(qreal)), SLOT(slotGridGuiChanged()));
+    connect(ui->angleRightAngleSelector, SIGNAL(angleChanged(qreal)), SLOT(slotGridGuiChanged()));
     connect(ui->cellSpacingSpinbox, SIGNAL(valueChanged(int)), SLOT(slotGridGuiChanged()));
 
     connect(ui->selectMainStyle, SIGNAL(currentIndexChanged(int)), SLOT(slotGridGuiChanged()));
@@ -147,25 +151,28 @@ void GridConfigWidget::setGridConfigImpl(const KisGridConfig &value)
     m_d->gridConfig = value;
     m_d->guiSignalsBlocked = true;
 
+    if (!m_d->gridConfig.offset().isNull()) {
+        ui->chkOffset->setChecked(true);
+    }
+
     ui->offsetAspectButton->setKeepAspectRatio(m_d->gridConfig.offsetAspectLocked());
     ui->spacingAspectButton->setKeepAspectRatio(m_d->gridConfig.spacingAspectLocked());
     ui->chkShowGrid->setChecked(m_d->gridConfig.showGrid());
-    ui->intHSpacing->setValue(m_d->gridConfig.spacing().x());
     ui->intHSpacing->setMaximum(std::numeric_limits<int>::max());
-    ui->intVSpacing->setValue(m_d->gridConfig.spacing().y());
     ui->intVSpacing->setMaximum(std::numeric_limits<int>::max());
+    ui->intHSpacing->setValue(m_d->gridConfig.spacing().x());
+    ui->intVSpacing->setValue(m_d->gridConfig.spacing().y());
     ui->intXOffset->setValue(m_d->gridConfig.offset().x());
     ui->intYOffset->setValue(m_d->gridConfig.offset().y());
     ui->intSubdivision->setValue(m_d->gridConfig.subdivision());
     ui->chkSnapToGrid->setChecked(m_d->gridConfig.snapToGrid());
-    ui->angleLeftSpinbox->setValue(m_d->gridConfig.angleLeft());
-    ui->angleRightSpinbox->setValue(m_d->gridConfig.angleRight());
+    ui->angleLeftAngleSelector->setAngle(m_d->gridConfig.angleLeft());
+    ui->angleRightAngleSelector->setAngle(m_d->gridConfig.angleRight());
     ui->cellSpacingSpinbox->setValue(m_d->gridConfig.cellSpacing());
 
-
     ui->selectMainStyle->setCurrentIndex(int(m_d->gridConfig.lineTypeMain()));
     ui->selectSubdivisionStyle->setCurrentIndex(int(m_d->gridConfig.lineTypeSubdivision()));
-    ui->gridTypeCombobox->setCurrentIndex(m_d->gridConfig.gridType());
+    ui->gridTypeCombobox->setCurrentIndex(int(m_d->gridConfig.gridType()));
 
     ui->colorMain->setColor(m_d->gridConfig.colorMain());
     ui->colorSubdivision->setColor(m_d->gridConfig.colorSubdivision());
@@ -220,10 +227,10 @@ KisGridConfig GridConfigWidget::fetchGuiGridConfig() const
     config.setOffset(pt);
 
     config.setSubdivision(ui->intSubdivision->value());
-    config.setAngleLeft(ui->angleLeftSpinbox->value());
-    config.setAngleRight(ui->angleRightSpinbox->value());
+    config.setAngleLeft(static_cast<int>(ui->angleLeftAngleSelector->angle()));
+    config.setAngleRight(static_cast<int>(ui->angleRightAngleSelector->angle()));
     config.setCellSpacing(ui->cellSpacingSpinbox->value());
-    config.setGridType(ui->gridTypeCombobox->currentIndex());
+    config.setGridType(KisGridConfig::GridType(ui->gridTypeCombobox->currentIndex()));
 
     config.setOffsetAspectLocked(ui->offsetAspectButton->keepAspectRatio());
     config.setSpacingAspectLocked(ui->spacingAspectButton->keepAspectRatio());
@@ -300,8 +307,8 @@ void GridConfigWidget::slotGridTypeChanged() {
     // specific isometric UI controls
     ui->leftAngleLabel->setVisible(!showRectangleControls);
     ui->rightAngleLabel->setVisible(!showRectangleControls);
-    ui->angleLeftSpinbox->setVisible(!showRectangleControls);
-    ui->angleRightSpinbox->setVisible(!showRectangleControls);
+    ui->angleLeftAngleSelector->setVisible(!showRectangleControls);
+    ui->angleRightAngleSelector->setVisible(!showRectangleControls);
     ui->cellSpacingLabel->setVisible(!showRectangleControls);
     ui->cellSpacingSpinbox->setVisible(!showRectangleControls);
 
