@@ -65,9 +65,9 @@ void HistogramDockerDock::setCanvas(KoCanvasBase * canvas)
 
     m_canvas = dynamic_cast<KisCanvas2*>(canvas);
     if (m_canvas) {
-        m_histogramWidget->setPaintDevice(m_canvas);
 
         m_imageIdleWatcher->setTrackedImage(m_canvas->image());
+        connect(m_imageIdleWatcher, &KisIdleWatcher::startedIdleMode, this, &HistogramDockerDock::updateHistogram, Qt::UniqueConnection);
 
         connect(m_canvas->image(), SIGNAL(sigImageUpdated(QRect)), this, SLOT(startUpdateCanvasProjection()), Qt::UniqueConnection);
         connect(m_canvas->image(), SIGNAL(sigColorSpaceChanged(const KoColorSpace*)), this, SLOT(sigColorSpaceChanged(const KoColorSpace*)), Qt::UniqueConnection);
@@ -79,7 +79,6 @@ void HistogramDockerDock::unsetCanvas()
 {
     setEnabled(false);
     m_canvas = 0;
-    m_histogramWidget->setPaintDevice(m_canvas);
     m_imageIdleWatcher->startCountdown();
 }
 
@@ -107,6 +106,6 @@ void HistogramDockerDock::sigColorSpaceChanged(const KoColorSpace */*cs*/)
 void HistogramDockerDock::updateHistogram()
 {
     if (isVisible()) {
-        m_histogramWidget->updateHistogram();
+        m_histogramWidget->updateHistogram(m_canvas);
     }
 }
