@@ -21,27 +21,41 @@
 #define RESOURCEMANAGER_H
 
 #include <QVariant>
-#include <KoResourceServer.h>
-#
-#include <kis_view_plugin.h>
+#include <QStringList>
+#include <QString>
 
-#include "dlg_create_bundle.h"
+#include <KoResourceServer.h>
+
+#include <KisActionPlugin.h>
 #include "KisResourceBundle.h"
 
-class ResourceManager : public KisViewPlugin
+#include "dlg_create_bundle.h"
+
+class ResourceManager : public KisActionPlugin
 {
     Q_OBJECT
 public:
     ResourceManager(QObject *parent, const QVariantList &);
-    virtual ~ResourceManager();
-    void saveBundle(const DlgCreateBundle &dlgCreateBundle);
+    ~ResourceManager() override;
+    KisResourceBundle *saveBundle(const DlgCreateBundle &dlgCreateBundle);
 
 private Q_SLOTS:
-    void slotImport();
     void slotCreateBundle();
     void slotManageBundles();
 
+    void slotImportBrushes();
+    void slotImportGradients();
+    void slotImportPalettes();
+    void slotImportPatterns();
+    void slotImportPresets();
+    void slotImportWorkspaces();
+    void slotImportBundles();
+
+
 private:
+
+    QStringList importResources(const QString &title, const QStringList &mimes) const;
+
     class Private;
     QScopedPointer<Private> d;
 };
