@@ -29,7 +29,6 @@
 #include <KoXmlReader.h>
 #include <KoPathShapeLoader.h>
 #include <KoShapeBackground.h>
-#include <KoPathShapeLoader.h>
 #include <KoEmbeddedDocumentSaver.h>
 #include <SvgSavingContext.h>
 #include <SvgLoadingContext.h>
@@ -38,6 +37,7 @@
 #include <SvgStyleParser.h>
 #include <SvgWriter.h>
 #include <SvgStyleWriter.h>
+#include <KisQPainterStateSaver.h>
 
 #include <klocalizedstring.h>
 
@@ -47,7 +47,6 @@
 #include <QPainter>
 #include <QFont>
 
-
 ArtisticTextShape::ArtisticTextShape()
     : m_path(0)
     , m_startOffset(0.0)
@@ -56,6 +55,7 @@ ArtisticTextShape::ArtisticTextShape()
     , m_defaultFont("ComicSans", 20)
 {
     setShapeId(ArtisticTextShapeID);
+    cacheGlyphOutlines();
     updateSizeAndPosition();
 }
 
@@ -66,24 +66,45 @@ ArtisticTextShape::~ArtisticTextShape()
     }
 }
 
-void ArtisticTextShape::paint(QPainter &painter, const KoViewConverter &converter, KoShapePaintingContext &paintContext)
+KoShape *ArtisticTextShape::cloneShape() const
 {
-    applyConversion(painter, converter);
+    ArtisticTextShape *clone = new ArtisticTextShape();
+    clone->m_ranges = m_ranges;
+    if (m_path) {
+        clone->m_path = static_cast<KoPathShape*>(m_path->cloneShape());
+    }
+    clone->m_charOutlines = m_charOutlines;
+    clone->m_startOffset = m_startOffset;
+    clone->m_outlineOrigin = m_outlineOrigin;
+    clone->m_outline = m_outline;
+    clone->m_baseline = m_baseline;
+    clone->m_textAnchor = m_textAnchor;
+    clone->m_charOffsets = m_charOffsets;
+    clone->m_charPositions = m_charPositions;
+    clone->m_textUpdateCounter = m_textUpdateCounter;
+    clone->m_defaultFont = m_defaultFont;
+    return clone;
+}
+
+void ArtisticTextShape::paint(QPainter &painter, KoShapePaintingContext &paintContext) const
+{
+    KisQPainterStateSaver saver(&painter);
+
     if (background()) {
-        background()->paint(painter, converter, paintContext, outline());
+        background()->paint(painter, paintContext, outline());
     }
 }
 
 void ArtisticTextShape::saveOdf(KoShapeSavingContext &context) const
 {
-    SvgWriter svgWriter(QList<KoShape *>() << const_cast<ArtisticTextShape *>(this), size());
+    SvgWriter svgWriter(QList<KoShape *>() << const_cast<ArtisticTextShape *>(this));
     QByteArray fileContent;
     QBuffer fileContentDevice(&fileContent);
     if (!fileContentDevice.open(QIODevice::WriteOnly)) {
         return;
     }
 
-    if (!svgWriter.save(fileContentDevice)) {
+    if (!svgWriter.save(fileContentDevice, size())) {
         qWarning() << "Could not write svg content";
         return;
     }
@@ -232,8 +253,11 @@ void ArtisticTextShape::createOutline()
 {
     // reset relevant data
     m_outline = QPainterPath();
+    m_charPositions.clear();
     m_charOffsets.clear();
-    cacheGlyphOutlines();
+
+    // calculate character positions in baseline coordinates
+    m_charPositions = calculateAbstractCharacterPositions();
 
     // the character index within the text shape
     int globalCharIndex = 0;
@@ -320,7 +344,8 @@ void ArtisticTextShape::createOutline()
         qreal rotation = 0.0;
         Q_FOREACH (const ArtisticTextRange &range, m_ranges) {
             const QString textRange = range.text();
-            for (int localCharIndex = 0; localCharIndex < m_charOutlines.size(); ++localCharIndex, ++globalCharIndex) {
+            const int localTextLength = textRange.length();
+            for (int localCharIndex = 0; localCharIndex < localTextLength; ++localCharIndex, ++globalCharIndex) {
                 const QPointF &charPos = m_charPositions[globalCharIndex];
                 if (range.hasRotation(localCharIndex)) {
                     rotation = range.rotation(localCharIndex);
@@ -371,11 +396,6 @@ QString ArtisticTextShape::plainText() const
     return allText;
 }
 
-int ArtisticTextShape::glyphsIndexSize() const
-{
-    return m_charOutlines.size();
-}
-
 QList<ArtisticTextRange> ArtisticTextShape::text() const
 {
     return m_ranges;
@@ -424,7 +444,7 @@ void ArtisticTextShape::setFont(int charIndex, int charCount, const QFont &font)
         return;
     }
 
-    if (charIndex == 0 && charCount == glyphsIndexSize()) {
+    if (charIndex == 0 && charCount == plainText().length()) {
         setFont(font);
         return;
     }
@@ -573,13 +593,13 @@ bool ArtisticTextShape::putOnPath(KoPathShape *path)
     m_path = path;
 
     // use the paths outline converted to document coordinates as the baseline
-    m_baseline = m_path->absoluteTransformation(0).map(m_path->outline());
+    m_baseline = m_path->absoluteTransformation().map(m_path->outline());
 
     // reset transformation
     setTransformation(QTransform());
     updateSizeAndPosition();
     // move to correct position
-    setAbsolutePosition(m_outlineOrigin, KoFlake::TopLeftCorner);
+    setAbsolutePosition(m_outlineOrigin, KoFlake::TopLeft);
     update();
 
     return true;
@@ -602,7 +622,7 @@ bool ArtisticTextShape::putOnPath(const QPainterPath &path)
     setTransformation(QTransform());
     updateSizeAndPosition();
     // move to correct position
-    setAbsolutePosition(m_outlineOrigin, KoFlake::TopLeftCorner);
+    setAbsolutePosition(m_outlineOrigin, KoFlake::TopLeft);
     update();
 
     return true;
@@ -653,7 +673,7 @@ QList<ArtisticTextRange> ArtisticTextShape::removeText(int charIndex, int charCo
         return extractedRanges;
     }
 
-    if (charIndex == 0 && charCount >= glyphsIndexSize()) {
+    if (charIndex == 0 && charCount >= plainText().length()) {
         beginTextUpdate();
         extractedRanges = m_ranges;
         m_ranges.clear();
@@ -738,7 +758,7 @@ void ArtisticTextShape::insertText(int charIndex, const QString &str)
     if (charIndex < 0) {
         // insert before first character
         charPos = CharIndex(0, 0);
-    } else if (charIndex >= glyphsIndexSize()) {
+    } else if (charIndex >= plainText().length()) {
         // insert after last character
         charPos = CharIndex(m_ranges.count() - 1, m_ranges.last().text().length());
     }
@@ -775,7 +795,7 @@ void ArtisticTextShape::insertText(int charIndex, const QList<ArtisticTextRange>
     if (charIndex < 0) {
         // insert before first character
         charPos = CharIndex(0, 0);
-    } else if (charIndex >= glyphsIndexSize()) {
+    } else if (charIndex >= plainText().length()) {
         // insert after last character
         charPos = CharIndex(m_ranges.count() - 1, m_ranges.last().text().length());
     }
@@ -885,13 +905,18 @@ QRectF ArtisticTextShape::charExtentsAt(int charIndex) const
     if (charIndex < 0 || isEmpty()) {
         charPos = CharIndex(0, 0);
     } else if (charPos.first < 0) {
-        charPos = CharIndex(m_ranges.count() - 1, glyphsIndexSize() - 1);
+        charPos = CharIndex(m_ranges.count() - 1, m_ranges.last().text().length() - 1);
     }
 
     if (charPos.first < m_ranges.size()) {
         const ArtisticTextRange &range = m_ranges.at(charPos.first);
         QFontMetrics metrics(range.font());
+#if QT_VERSION >= QT_VERSION_CHECK(5,11,0)
+        QChar c = range.text().at(charPos.second);
+        int w = metrics.horizontalAdvance(c);
+#else
         int w = metrics.charWidth(range.text(), charPos.second);
+#endif
         return QRectF(0, 0, w, metrics.height());
     }
 
@@ -900,7 +925,7 @@ QRectF ArtisticTextShape::charExtentsAt(int charIndex) const
 
 void ArtisticTextShape::updateSizeAndPosition(bool global)
 {
-    QTransform shapeTransform = absoluteTransformation(0);
+    QTransform shapeTransform = absoluteTransformation();
 
     // determine baseline position in document coordinates
     QPointF oldBaselinePosition = shapeTransform.map(QPointF(0, baselineOffset()));
@@ -948,50 +973,17 @@ void ArtisticTextShape::updateSizeAndPosition(bool global)
 void ArtisticTextShape::cacheGlyphOutlines()
 {
     m_charOutlines.clear();
-    m_charPositions.clear();
-    m_charPositions.resize(1);
-    QPointF charPos(0, 0);
-    int globalCharIndex = 0;
-    int glyph_count = 0;
+
     Q_FOREACH (const ArtisticTextRange &range, m_ranges) {
         const QString rangeText = range.text();
         const QFont rangeFont(range.font(), &m_paintDevice);
-        QFontMetrics metrics(rangeFont);
-        QTextLayout layout;
-        layout.setText(rangeText);
-        layout.beginLayout();
-        int leading = metrics.leading();
-        qreal height = QPointF().y();
-        while (1) {
-            QTextLine line = layout.createLine();
-            if (!line.isValid())
-                break;
-            height += leading;
-            line.setPosition(QPointF(QPointF().x(), height));
-            height += line.height();
-        }
-        layout.endLayout();
-        QList<QGlyphRun> runs = layout.glyphRuns();
-
-
-        foreach (QGlyphRun run, runs) {
-            QRawFont rawFont(run.rawFont());
-            QVector<quint32> indexes =run.glyphIndexes();
-            QVector<QPointF> positions = run.positions();
-            glyph_count += positions.size();
-            m_charPositions.resize(glyph_count + 1);
-            for (int i =0; i< indexes.size(); i++, ++globalCharIndex){
-                QPainterPath path;
-                path = rawFont.pathForGlyph(indexes.at(i));
-                m_charOutlines.append(path);
-                qreal yposiotn = positions[i].ry() - positions[0].ry();
-                charPos = QPointF(positions.at(i).x() , yposiotn);
-                m_charPositions[globalCharIndex] = charPos;
-            }
+        const int textLength = rangeText.length();
+        for (int charIdx = 0; charIdx < textLength; ++charIdx) {
+            QPainterPath charOutline;
+            charOutline.addText(QPointF(), rangeFont, rangeText[charIdx]);
+            m_charOutlines.append(charOutline);
         }
     }
-    m_charPositions[globalCharIndex] = charPos;
-
 }
 
 void ArtisticTextShape::shapeChanged(ChangeType type, KoShape *shape)
@@ -1007,7 +999,7 @@ void ArtisticTextShape::shapeChanged(ChangeType type, KoShape *shape)
         } else {
             update();
             // use the paths outline converted to document coordinates as the baseline
-            m_baseline = m_path->absoluteTransformation(0).map(m_path->outline());
+            m_baseline = m_path->absoluteTransformation().map(m_path->outline());
             updateSizeAndPosition(true);
             update();
         }
@@ -1050,6 +1042,7 @@ void ArtisticTextShape::finishTextUpdate()
         return;
     }
 
+    cacheGlyphOutlines();
     updateSizeAndPosition();
     update();
     notifyChanged();
@@ -1083,9 +1076,9 @@ bool ArtisticTextShape::saveSvg(SvgSavingContext &context)
 
     // check if we are set on a path
     if (layout() == ArtisticTextShape::Straight) {
-        context.shapeWriter().addAttributePt("x", anchorOffset);
-        context.shapeWriter().addAttributePt("y", baselineOffset());
-        context.shapeWriter().addAttribute("transform", SvgUtil::transformToString(transformation()));
+        context.shapeWriter().addAttribute("x", anchorOffset);
+        context.shapeWriter().addAttribute("y", baselineOffset());
+        SvgUtil::writeTransformAttributeLazy("transform", transformation(), context.shapeWriter());
         Q_FOREACH (const ArtisticTextRange &range, formattedText) {
             saveSvgTextRange(range, context, !hasSingleRange, baselineOffset());
         }
@@ -1095,7 +1088,7 @@ bool ArtisticTextShape::saveSvg(SvgSavingContext &context)
         QString id = context.createUID("baseline");
         context.styleWriter().startElement("path");
         context.styleWriter().addAttribute("id", id);
-        context.styleWriter().addAttribute("d", baselineShape->toString(baselineShape->absoluteTransformation(0) * context.userSpaceTransform()));
+        context.styleWriter().addAttribute("d", baselineShape->toString(baselineShape->absoluteTransformation() * context.userSpaceTransform()));
         context.styleWriter().endElement();
 
         context.shapeWriter().startElement("textPath");
@@ -1119,7 +1112,7 @@ bool ArtisticTextShape::saveSvg(SvgSavingContext &context)
 void ArtisticTextShape::saveSvgFont(const QFont &font, SvgSavingContext &context)
 {
     context.shapeWriter().addAttribute("font-family", font.family());
-    context.shapeWriter().addAttributePt("font-size", font.pointSizeF());
+    context.shapeWriter().addAttribute("font-size", font.pointSizeF());
 
     if (font.bold()) {
         context.shapeWriter().addAttribute("font-weight", "bold");
@@ -1252,7 +1245,7 @@ bool ArtisticTextShape::loadSvg(const KoXmlElement &textElement, SvgLoadingConte
 
                 path->setSize(newSize);
                 path->setPosition(newPosition);
-                path->applyAbsoluteTransformation(SvgUtil::parseTransform(p.attribute("transform")));
+                path->applyAbsoluteTransformation(context.currentGC()->matrix);
             }
         } else {
             path = dynamic_cast<KoPathShape *>(context.shapeById(href));
@@ -1295,7 +1288,7 @@ bool ArtisticTextShape::loadSvg(const KoXmlElement &textElement, SvgLoadingConte
             if (pathInDocument) {
                 putOnPath(path);
             } else {
-                putOnPath(path->absoluteTransformation(0).map(path->outline()));
+                putOnPath(path->absoluteTransformation().map(path->outline()));
                 delete path;
             }
 
@@ -1387,8 +1380,11 @@ ArtisticTextRange ArtisticTextShape::createTextRange(const QString &text, Artist
     }
 
     range.setRotations(context.rotations(textLength));
+
+#if 0
     range.setLetterSpacing(gc->letterSpacing);
     range.setWordSpacing(gc->wordSpacing);
+
     if (gc->baselineShift == "sub") {
         range.setBaselineShift(ArtisticTextRange::Sub);
     } else if (gc->baselineShift == "super") {
@@ -1401,6 +1397,7 @@ ArtisticTextRange ArtisticTextShape::createTextRange(const QString &text, Artist
             range.setBaselineShift(ArtisticTextRange::Length, value);
         }
     }
+#endif
 
     //range.printDebug();
 
