@@ -66,16 +66,28 @@ KoPatternGenerator::KoPatternGenerator() : KisGenerator(id(), KoID("basic"), i18
     setSupportsPainting(true);
 }
 
-KisFilterConfigurationSP KoPatternGenerator::factoryConfiguration() const
+KisFilterConfigurationSP KoPatternGenerator::defaultConfiguration() const
 {
-    KisFilterConfigurationSP config = new KisFilterConfiguration("pattern", 1);
+    KisFilterConfigurationSP config = factoryConfiguration();
 
     QVariant v;
     v.setValue(QString("Grid01.pat"));
     config->setProperty("pattern", v);
 
-//    v.setValue(KoColor());
-//    config->setProperty("color", v);
+    config->setProperty("transform_shear_x", QVariant::fromValue(0.0));
+    config->setProperty("transform_shear_y", QVariant::fromValue(0.0));
+
+    config->setProperty("transform_scale_x", QVariant::fromValue(1.0));
+    config->setProperty("transform_scale_y", QVariant::fromValue(1.0));
+
+    config->setProperty("transform_rotation_x", QVariant::fromValue(0.0));
+    config->setProperty("transform_rotation_y", QVariant::fromValue(0.0));
+    config->setProperty("transform_rotation_z", QVariant::fromValue(0.0));
+
+    config->setProperty("transform_offset_x", QVariant::fromValue(0));
+    config->setProperty("transform_offset_y", QVariant::fromValue(0));
+
+    config->setProperty("transform_keep_scale_aspect", QVariant::fromValue(true));
 
     return config;
 }
@@ -100,12 +112,19 @@ void KoPatternGenerator::generate(KisProcessingInformation dstInfo,
     QString patternName = config->getString("pattern", "Grid01.pat");
     KoResourceServer<KoPattern> *rserver = KoResourceServerProvider::instance()->patternServer();
     KoPattern *pattern = rserver->resourceByName(patternName);
+    QTransform transform;
 
-//    KoColor c = config->getColor("color");
+    transform.shear(config->getDouble("transform_shear_x", 0.0), config->getDouble("transform_shear_y", 0.0));
+
+    transform.scale(config->getDouble("transform_scale_x", 1.0), config->getDouble("transform_scale_y", 1.0));
+    transform.rotate(config->getDouble("transform_rotation_x", 0.0), Qt::XAxis);
+    transform.rotate(config->getDouble("transform_rotation_y", 0.0), Qt::YAxis);
+    transform.rotate(config->getDouble("transform_rotation_z", 0.0), Qt::ZAxis);
+
+    transform.translate(config->getInt("transform_offset_x", 0), config->getInt("transform_offset_y", 0));
 
     KisFillPainter gc(dst);
     gc.setPattern(pattern);
-//    gc.setPaintColor(c);
     gc.setProgress(progressUpdater);
     gc.setChannelFlags(config->channelFlags());
     gc.setOpacity(OPACITY_OPAQUE_U8);
@@ -113,7 +132,13 @@ void KoPatternGenerator::generate(KisProcessingInformation dstInfo,
     gc.setWidth(size.width());
     gc.setHeight(size.height());
     gc.setFillStyle(KisFillPainter::FillStylePattern);
-    gc.fillRect(QRect(dstInfo.topLeft(), size), pattern);
+    /**
+     * HACK ALERT: using "no-compose" version of `fillRect` discards all the opacity,
+     * selection, and channel flags options. Though it doesn't seem that we have a any
+     * GUI in Krita that actually passes a selection to the generator itself. Fill
+     * layers apply their settings on a later stage of the compositing pipeline.
+     */
+    gc.fillRectNoCompose(QRect(dstInfo.topLeft(), size), pattern, transform);
     gc.end();
 
 }
