@@ -43,7 +43,7 @@
 
 #include <kis_icon.h>
 #include "kis_brush_server.h"
-#include "widgets/kis_slider_spin_box.h"
+#include "kis_slider_spin_box.h"
 #include "widgets/kis_multipliers_double_slider_spinbox.h"
 #include "kis_spacing_selection_widget.h"
 #include "kis_signals_blocker.h"
@@ -55,6 +55,7 @@
 
 #include "kis_global.h"
 #include "kis_gbr_brush.h"
+#include "kis_png_brush.h"
 #include "kis_debug.h"
 #include "kis_image.h"
 
@@ -120,16 +121,12 @@ KisPredefinedBrushChooser::KisPredefinedBrushChooser(QWidget *parent, const char
 
     QObject::connect(brushSizeSpinBox, SIGNAL(valueChanged(qreal)), this, SLOT(slotSetItemSize(qreal)));
 
-    brushRotationSpinBox->setRange(0, 360, 0);
-    brushRotationSpinBox->setValue(0);
-    brushRotationSpinBox->setSuffix(QChar(Qt::Key_degree));
-    QObject::connect(brushRotationSpinBox, SIGNAL(valueChanged(qreal)), this, SLOT(slotSetItemRotation(qreal)));
+    brushRotationAngleSelector->setDecimals(0);
+    QObject::connect(brushRotationAngleSelector, SIGNAL(angleChanged(qreal)), this, SLOT(slotSetItemRotation(qreal)));
 
     brushSpacingSelectionWidget->setSpacing(true, 1.0);
     connect(brushSpacingSelectionWidget, SIGNAL(sigSpacingChanged()), SLOT(slotSpacingChanged()));
 
-    QObject::connect(useColorAsMaskCheckbox, SIGNAL(toggled(bool)), this, SLOT(slotSetItemUseColorAsMask(bool)));
-
     KisBrushResourceServer* rServer = KisBrushServer::instance()->brushServer();
     QSharedPointer<KisBrushResourceServerAdapter> adapter(new KisBrushResourceServerAdapter(rServer));
 
@@ -176,6 +173,47 @@ KisPredefinedBrushChooser::KisPredefinedBrushChooser(QWidget *parent, const char
     resetBrushButton->setToolTip(i18n("Reloads Spacing from file\nSets Scale to 1.0\nSets Rotation to 0.0"));
     connect(resetBrushButton, SIGNAL(clicked()), SLOT(slotResetBrush()));
 
+    intAdjustmentMidPoint->setRange(0, 255);
+    intAdjustmentMidPoint->setPageStep(10);
+    intAdjustmentMidPoint->setSingleStep(1);
+    intAdjustmentMidPoint->setPrefix(i18nc("@label:slider", "Neutral point: "));
+
+    intBrightnessAdjustment->setRange(-100, 100);
+    intBrightnessAdjustment->setPageStep(10);
+    intBrightnessAdjustment->setSingleStep(1);
+    intBrightnessAdjustment->setSuffix("%");
+    intBrightnessAdjustment->setPrefix(i18nc("@label:slider", "Brightness: "));
+
+    intContrastAdjustment->setRange(-100, 100);
+    intContrastAdjustment->setPageStep(10);
+    intContrastAdjustment->setSingleStep(1);
+    intContrastAdjustment->setSuffix("%");
+    intContrastAdjustment->setPrefix(i18nc("@label:slider", "Contrast: "));
+
+    btnResetAdjustments->setToolTip(i18nc("@info:tooltip", "Resets all the adjustments to default values:\n Neutral Point: 127\n Brightness: 0%\n Contrast: 0%"));
+    connect(btnResetAdjustments, SIGNAL(clicked()), SLOT(slotResetAdjustments()));
+
+    cmbBrushMode->addItem(i18n("Alpha Mask"));
+    cmbBrushMode->addItem(i18n("Color Image"));
+    cmbBrushMode->addItem(i18n("Lightness Map"));
+    cmbBrushMode->addItem(i18n("Gradient Map"));
+    cmbBrushMode->setItemData(int(ALPHAMASK), i18nc("@info:tooltip", "Luminosity of the brush tip image is used as alpha channel for the stroke"), Qt::ToolTipRole);
+    cmbBrushMode->setItemData(int(IMAGESTAMP), i18nc("@info:tooltip", "The brush tip image is painted as it is"), Qt::ToolTipRole);
+    cmbBrushMode->setItemData(int(LIGHTNESSMAP), i18nc("@info:tooltip", "Luminosity of the brush tip image is used as lightness correction for the painting color. Alpha channel of the brush tip image is used as alpha for the final stroke"), Qt::ToolTipRole);
+    cmbBrushMode->setItemData(int(GRADIENTMAP), i18nc("@info:tooltip", "The brush tip maps its value to the currently selected gradient. Alpha channel of the brush tip image is used as alpha for the final stroke"), Qt::ToolTipRole);
+
+    connect(cmbBrushMode, SIGNAL(currentIndexChanged(int)), SLOT(slotUpdateBrushAdjustmentsState()));
+    connect(cmbBrushMode, SIGNAL(currentIndexChanged(int)), SLOT(slotWriteBrushMode()));
+    connect(cmbBrushMode, SIGNAL(currentIndexChanged(int)), SLOT(slotUpdateResetBrushAdjustmentsButtonState()));
+
+    connect(intAdjustmentMidPoint, SIGNAL(valueChanged(int)), SLOT(slotWriteBrushAdjustments()));
+    connect(intBrightnessAdjustment, SIGNAL(valueChanged(int)), SLOT(slotWriteBrushAdjustments()));
+    connect(intContrastAdjustment, SIGNAL(valueChanged(int)), SLOT(slotWriteBrushAdjustments()));
+
+    connect(intAdjustmentMidPoint, SIGNAL(valueChanged(int)), SLOT(slotUpdateResetBrushAdjustmentsButtonState()));
+    connect(intBrightnessAdjustment, SIGNAL(valueChanged(int)), SLOT(slotUpdateResetBrushAdjustmentsButtonState()));
+    connect(intContrastAdjustment, SIGNAL(valueChanged(int)), SLOT(slotUpdateResetBrushAdjustmentsButtonState()));
+
     updateBrushTip(m_itemChooser->currentResource());
 }
 
@@ -229,6 +267,13 @@ void KisPredefinedBrushChooser::slotResetBrush()
         brush->setScale(1.0);
         brush->setAngle(0.0);
 
+        if (KisColorfulBrush *colorfulBrush = dynamic_cast<KisColorfulBrush*>(m_brush.data())) {
+            colorfulBrush->setBrushApplication(IMAGESTAMP);
+            colorfulBrush->setAdjustmentMidPoint(127);
+            colorfulBrush->setBrightnessAdjustment(0.0);
+            colorfulBrush->setContrastAdjustment(0.0);
+        }
+
         updateBrushTip(brush);
         emit sigBrushChanged();
     }
@@ -268,17 +313,6 @@ void KisPredefinedBrushChooser::slotSpacingChanged()
     }
 }
 
-void KisPredefinedBrushChooser::slotSetItemUseColorAsMask(bool useColorAsMask)
-{
-    KIS_SAFE_ASSERT_RECOVER_RETURN(m_brush);
-
-    KisGbrBrush *brush = dynamic_cast<KisGbrBrush *>(m_brush.data());
-    if (brush) {
-        brush->setUseColorAsMask(useColorAsMask);
-        emit sigBrushChanged();
-    }
-}
-
 void KisPredefinedBrushChooser::slotOpenStampBrush()
 {
     if(!m_stampBrushWidget) {
@@ -333,7 +367,7 @@ void KisPredefinedBrushChooser::updateBrushTip(KoResource * resource, bool isCha
         } else if (m_brush->brushType() == MASK) {
             brushTypeString = i18n("Mask");
         } else if (m_brush->brushType() == IMAGE) {
-            brushTypeString = i18n("GBR");
+            brushTypeString = i18n("Image");
         } else if (m_brush->brushType() == PIPE_MASK ) {
             brushTypeString = i18n("Animated Mask"); // GIH brush
 
@@ -359,7 +393,7 @@ void KisPredefinedBrushChooser::updateBrushTip(KoResource * resource, bool isCha
         // this will set the brush's model data to keep what it currently has for size, spacing, etc.
         if (preserveBrushPresetSettings->isChecked() && !isChangingBrushPresets) {
             m_brush->setAutoSpacing(brushSpacingSelectionWidget->autoSpacingActive(), brushSpacingSelectionWidget->autoSpacingCoeff());
-            m_brush->setAngle(brushRotationSpinBox->value() * M_PI / 180);
+            m_brush->setAngle(brushRotationAngleSelector->angle() * M_PI / 180);
             m_brush->setSpacing(brushSpacingSelectionWidget->spacing());
             m_brush->setUserEffectiveSize(brushSizeSpinBox->value());
         }
@@ -368,20 +402,121 @@ void KisPredefinedBrushChooser::updateBrushTip(KoResource * resource, bool isCha
                                 m_brush->autoSpacingActive() ?
                                 m_brush->autoSpacingCoeff() : m_brush->spacing());
 
-        brushRotationSpinBox->setValue(m_brush->angle() * 180 / M_PI);
+        brushRotationAngleSelector->setAngle(m_brush->angle() * 180 / M_PI);
         brushSizeSpinBox->setValue(m_brush->width() * m_brush->scale());
 
-        // useColorAsMask support is only in gimp brush so far
-        bool prevColorAsMaskState = useColorAsMaskCheckbox->isChecked();
-        KisGbrBrush *gimpBrush = dynamic_cast<KisGbrBrush*>(m_brush.data());
-        if (gimpBrush) {
-            useColorAsMaskCheckbox->setChecked(gimpBrush->useColorAsMask() || prevColorAsMaskState);
-            gimpBrush->setUseColorAsMask(prevColorAsMaskState);
-        }
-        useColorAsMaskCheckbox->setEnabled(m_brush->hasColor() && gimpBrush);
-
         emit sigBrushChanged();
     }
+
+    slotUpdateBrushModeButtonsState();
+}
+
+#include "kis_scaling_size_brush.h"
+
+void KisPredefinedBrushChooser::slotUpdateBrushModeButtonsState()
+{
+    KisColorfulBrush *colorfulBrush = dynamic_cast<KisColorfulBrush*>(m_brush.data());
+    const bool modeSwitchEnabled =
+        m_hslBrushTipEnabled && colorfulBrush && colorfulBrush->isImageType();
+
+    if (modeSwitchEnabled) {
+        cmbBrushMode->setCurrentIndex(int(colorfulBrush->brushApplication()));
+
+        {
+            // sliders emit update signals when modified from the code
+            KisSignalsBlocker b(intAdjustmentMidPoint, intBrightnessAdjustment, intContrastAdjustment);
+            intAdjustmentMidPoint->setValue(colorfulBrush->adjustmentMidPoint());
+            intBrightnessAdjustment->setValue(qRound(colorfulBrush->brightnessAdjustment() * 100.0));
+            intContrastAdjustment->setValue(qRound(colorfulBrush->contrastAdjustment() * 100.0));
+        }
+
+        intAdjustmentMidPoint->setToolTip(i18nc("@info:tooltip", "Luminosity value of the brush that will not change the painting color. All brush pixels darker than neutral point will paint with darker color, pixels lighter than neutral point — lighter."));
+        intBrightnessAdjustment->setToolTip(i18nc("@info:tooltip", "Brightness correction for the brush"));
+        intContrastAdjustment->setToolTip(i18nc("@info:tooltip", "Contrast correction for the brush"));
+        grpBrushMode->setToolTip("");
+    } else {
+
+        {
+            // sliders emit update signals when modified from the code
+            KisSignalsBlocker b(intAdjustmentMidPoint, intBrightnessAdjustment, intContrastAdjustment);
+            intAdjustmentMidPoint->setValue(127);
+            intBrightnessAdjustment->setValue(0);
+            intContrastAdjustment->setValue(0);
+        }
+
+        intAdjustmentMidPoint->setToolTip("");
+        intBrightnessAdjustment->setToolTip("");
+        intContrastAdjustment->setToolTip("");
+        if (m_hslBrushTipEnabled) {
+            grpBrushMode->setToolTip(i18nc("@info:tooltip", "The selected brush tip does not have color channels. The brush will work in \"Mask\" mode."));
+        }
+        else {
+            grpBrushMode->setToolTip(i18nc("@info:tooltip", "The selected brush engine does not support \"Color\" or \"Lightness\" modes. The brush will work in \"Mask\" mode."));
+        }
+
+    }
+
+
+    grpBrushMode->setEnabled(modeSwitchEnabled);
+    slotUpdateBrushAdjustmentsState();
+    slotUpdateResetBrushAdjustmentsButtonState();
+}
+
+void KisPredefinedBrushChooser::slotUpdateBrushAdjustmentsState()
+{
+    const bool adjustmentsEnabled = (cmbBrushMode->currentIndex() == LIGHTNESSMAP) ||
+                    (cmbBrushMode->currentIndex() == GRADIENTMAP);
+    intAdjustmentMidPoint->setEnabled(adjustmentsEnabled);
+    intBrightnessAdjustment->setEnabled(adjustmentsEnabled);
+    intContrastAdjustment->setEnabled(adjustmentsEnabled);
+}
+
+void KisPredefinedBrushChooser::slotUpdateResetBrushAdjustmentsButtonState()
+{
+    const bool adjustmentsEnabled = (cmbBrushMode->currentIndex() == LIGHTNESSMAP) ||
+                    (cmbBrushMode->currentIndex() == GRADIENTMAP);
+
+    const bool adjustmentsDefault =
+            intAdjustmentMidPoint->value() == 127 &&
+            intBrightnessAdjustment->value() == 0 &&
+            intContrastAdjustment->value() == 0;
+
+    btnResetAdjustments->setEnabled(!adjustmentsDefault && adjustmentsEnabled);
+}
+
+void KisPredefinedBrushChooser::slotWriteBrushMode()
+{
+    KisColorfulBrush *colorfulBrush = dynamic_cast<KisColorfulBrush*>(m_brush.data());
+    if (!colorfulBrush) return;
+
+    colorfulBrush->setBrushApplication(enumBrushApplication(cmbBrushMode->currentIndex()));
+
+    emit sigBrushChanged();
+}
+
+void KisPredefinedBrushChooser::slotWriteBrushAdjustments()
+{
+    KisColorfulBrush *colorfulBrush = dynamic_cast<KisColorfulBrush*>(m_brush.data());
+    if (!colorfulBrush) return;
+
+    {
+        // sliders emit update signals when modified from the code
+        KisSignalsBlocker b(intAdjustmentMidPoint, intBrightnessAdjustment, intContrastAdjustment);
+        colorfulBrush->setAdjustmentMidPoint(quint8(intAdjustmentMidPoint->value()));
+        colorfulBrush->setBrightnessAdjustment(intBrightnessAdjustment->value() / 100.0);
+        colorfulBrush->setContrastAdjustment(intContrastAdjustment->value() / 100.0);
+    }
+
+    emit sigBrushChanged();
+}
+
+void KisPredefinedBrushChooser::slotResetAdjustments()
+{
+    intAdjustmentMidPoint->setValue(127);
+    intBrightnessAdjustment->setValue(0);
+    intContrastAdjustment->setValue(0);
+
+    slotWriteBrushAdjustments();
 }
 
 void KisPredefinedBrushChooser::slotNewPredefinedBrush(KoResource *resource)
@@ -406,6 +541,17 @@ void KisPredefinedBrushChooser::setImage(KisImageWSP image)
     m_image = image;
 }
 
+void KisPredefinedBrushChooser::setHSLBrushTipEnabled(bool value)
+{
+    m_hslBrushTipEnabled = value;
+}
+
+bool KisPredefinedBrushChooser::hslBrushTipEnabled() const
+{
+    return m_hslBrushTipEnabled;
+}
+
+
 void KisPredefinedBrushChooser::slotImportNewBrushResource() {
     m_itemChooser->slotButtonClicked(KoResourceItemChooser::Button_Import);
 }
