@@ -54,7 +54,7 @@
 #include "kis_resources_snapshot.h"
 #include "commands_new/KisMergeLabeledLayersCommand.h"
 #include <kis_color_filter_combo.h>
-
+#include <KisAngleSelector.h>
 
 #include <processing/fill_processing_visitor.h>
 #include <kis_processing_applicator.h>
@@ -264,11 +264,10 @@ QWidget* KisToolFill::createOptionWidget()
     m_checkUsePattern->setToolTip(i18n("When checked do not use the foreground color, but the pattern selected to fill with"));
 
     QLabel *lbl_patternRotation = new QLabel(i18n("Rotate:"), widget);
-    m_sldPatternRotate = new KisDoubleSliderSpinBox(widget);
-    m_sldPatternRotate->setObjectName("patternrotate");
-    m_sldPatternRotate->setRange(0, 360, 2);
-    m_sldPatternRotate->setSingleStep(1.0);
-    m_sldPatternRotate->setSuffix(QChar(Qt::Key_degree));
+    m_angleSelectorPatternRotate = new KisAngleSelector(widget);
+    m_angleSelectorPatternRotate->setFlipOptionsMode(KisAngleSelector::FlipOptionsMode_MenuButton);
+    m_angleSelectorPatternRotate->setIncreasingDirection(KisAngleGauge::IncreasingDirection_Clockwise);
+    m_angleSelectorPatternRotate->setObjectName("patternrotate");
 
     QLabel *lbl_patternScale = new QLabel(i18n("Scale:"), widget);
     m_sldPatternScale = new KisDoubleSliderSpinBox(widget);
@@ -313,7 +312,7 @@ QWidget* KisToolFill::createOptionWidget()
 
     connect (m_cmbSampleLayersMode   , SIGNAL(currentIndexChanged(int)), this, SLOT(slotSetSampleLayers(int)));
     connect (m_cmbSelectedLabels          , SIGNAL(selectedColorsChanged()), this, SLOT(slotSetSelectedColorLabels()));
-    connect (m_sldPatternRotate  , SIGNAL(valueChanged(qreal)), this, SLOT(slotSetPatternRotation(qreal)));
+    connect (m_angleSelectorPatternRotate  , SIGNAL(angleChanged(qreal)), this, SLOT(slotSetPatternRotation(qreal)));
     connect (m_sldPatternScale   , SIGNAL(valueChanged(qreal)), this, SLOT(slotSetPatternScale(qreal)));
 
     addOptionWidgetOption(m_checkUseFastMode, lbl_fastMode);
@@ -327,7 +326,7 @@ QWidget* KisToolFill::createOptionWidget()
     addOptionWidgetOption(m_cmbSelectedLabels, lbl_cmbLabel);
     addOptionWidgetOption(m_checkUsePattern, lbl_usePattern);
 
-    addOptionWidgetOption(m_sldPatternRotate, lbl_patternRotation);
+    addOptionWidgetOption(m_angleSelectorPatternRotate, lbl_patternRotation);
     addOptionWidgetOption(m_sldPatternScale, lbl_patternScale);
 
     updateGUI();
@@ -365,7 +364,7 @@ QWidget* KisToolFill::createOptionWidget()
     // m_sampleLayersMode is set manually above
     // selectedColors are also set manually
 
-    m_sldPatternRotate->setValue(m_configGroup.readEntry("patternRotate", 0.0));
+    m_angleSelectorPatternRotate->setAngle(m_configGroup.readEntry("patternRotate", 0.0));
     m_sldPatternScale->setValue(m_configGroup.readEntry("patternScale", 100.0));
 
     activateConnectionsToImage();
@@ -385,7 +384,7 @@ void KisToolFill::updateGUI()
     m_sizemodWidget->setEnabled(!selectionOnly && useAdvancedMode);
     m_featherWidget->setEnabled(!selectionOnly && useAdvancedMode);
     m_checkUsePattern->setEnabled(useAdvancedMode);
-    m_sldPatternRotate->setEnabled((m_checkUsePattern->isChecked() && useAdvancedMode));
+    m_angleSelectorPatternRotate->setEnabled((m_checkUsePattern->isChecked() && useAdvancedMode));
     m_sldPatternScale->setEnabled((m_checkUsePattern->isChecked() && useAdvancedMode));
 
     m_cmbSampleLayersMode->setEnabled(!selectionOnly && useAdvancedMode);
@@ -466,7 +465,7 @@ void KisToolFill::slotSetUsePattern(bool state)
 {
     m_usePattern = state;
     m_sldPatternScale->setEnabled(state);
-    m_sldPatternRotate->setEnabled(state);
+    m_angleSelectorPatternRotate->setEnabled(state);
     m_configGroup.writeEntry("usePattern", state);
 }
 
