@@ -127,7 +127,9 @@ struct KisFreeTransformStrategy::Private
 
     ToolTransformArgs clickArgs;
     QPointF clickPos;
-    bool isTransforming;
+    QTransform clickTransform;
+
+    bool isTransforming {false};
 
     QCursor getScaleCursor(const QPointF &handlePt);
     QCursor getShearCursor(const QPointF &start, const QPointF &end);
@@ -163,8 +165,11 @@ void KisFreeTransformStrategy::Private::recalculateTransformedHandles()
     transformedHandles.bottomRight = transform.map(transaction.originalBottomRight());
 }
 
-void KisFreeTransformStrategy::setTransformFunction(const QPointF &mousePos, bool perspectiveModifierActive)
+void KisFreeTransformStrategy::setTransformFunction(const QPointF &mousePos, bool perspectiveModifierActive, bool shiftModifierActive, bool altModifierActive)
 {
+    Q_UNUSED(shiftModifierActive);
+    Q_UNUSED(altModifierActive);
+
     if (perspectiveModifierActive && !m_d->transaction.shouldAvoidPerspectiveTransform()) {
         m_d->function = PERSPECTIVE;
         return;
@@ -387,6 +392,9 @@ bool KisFreeTransformStrategy::beginPrimaryAction(const QPointF &pt)
     m_d->clickArgs = m_d->currentArgs;
     m_d->clickPos = pt;
 
+    KisTransformUtils::MatricesPack m(m_d->clickArgs);
+    m_d->clickTransform = m.finalTransform();
+
     return true;
 }
 
@@ -482,40 +490,44 @@ void KisFreeTransformStrategy::continuePrimaryAction(const QPointF &mousePos,
     case BOTTOMSCALE: {
         QPointF staticPoint;
         QPointF movingPoint;
-        qreal extraSign;
 
         if (m_d->function == TOPSCALE) {
             staticPoint = m_d->transaction.originalMiddleBottom();
             movingPoint = m_d->transaction.originalMiddleTop();
-            extraSign = -1.0;
         } else {
             staticPoint = m_d->transaction.originalMiddleTop();
             movingPoint = m_d->transaction.originalMiddleBottom();
-            extraSign = 1.0;
         }
 
+        QPointF staticPointInView = m_d->clickTransform.map(staticPoint);
+        const QPointF movingPointInView = m_d->clickTransform.map(movingPoint);
+
+        const QPointF projNormVector =
+            KisAlgebra2D::normalize(movingPointInView - staticPointInView);
+
+        const qreal projLength =
+            KisAlgebra2D::dotProduct(mousePos - staticPointInView, projNormVector);
+
+        const QPointF targetMovingPointInView = staticPointInView + projNormVector * projLength;
+
         // override scale static point if it is locked
-        if ((m_d->currentArgs.transformAroundRotationCenter() ^ altModifierActive) &&
+        if ((m_d->clickArgs.transformAroundRotationCenter() ^ altModifierActive) &&
             !qFuzzyCompare(anchorPoint.y(), movingPoint.y())) {
 
             staticPoint = anchorPoint;
+            staticPointInView = m_d->clickTransform.map(staticPoint);
         }
 
-        QPointF mouseImagePos = m_d->transform.inverted().map(mousePos);
-
-        qreal sign = mouseImagePos.y() <= staticPoint.y() ? -extraSign : extraSign;
-        m_d->currentArgs.setScaleY(sign * m_d->currentArgs.scaleY());
-
-        QPointF staticPointInView = m_d->transform.map(staticPoint);
-
-        qreal dist = kisDistance(staticPointInView, mousePos);
-
         GSL::ScaleResult1D result =
             GSL::calculateScaleY(m_d->currentArgs,
                                  staticPoint,
                                  staticPointInView,
                                  movingPoint,
-                                 dist);
+                                 targetMovingPointInView);
+
+        if (!result.isValid) {
+            break;
+        }
 
         if (shiftModifierActive ||  m_d->currentArgs.keepAspectRatio()) {
             qreal aspectRatio = m_d->clickArgs.scaleX() / m_d->clickArgs.scaleY();
@@ -531,41 +543,44 @@ void KisFreeTransformStrategy::continuePrimaryAction(const QPointF &mousePos,
     case RIGHTSCALE: {
         QPointF staticPoint;
         QPointF movingPoint;
-        qreal extraSign;
 
         if (m_d->function == LEFTSCALE) {
-
             staticPoint = m_d->transaction.originalMiddleRight();
             movingPoint = m_d->transaction.originalMiddleLeft();
-            extraSign = -1.0;
         } else {
             staticPoint = m_d->transaction.originalMiddleLeft();
             movingPoint = m_d->transaction.originalMiddleRight();
-            extraSign = 1.0;
         }
 
+        QPointF staticPointInView = m_d->clickTransform.map(staticPoint);
+        const QPointF movingPointInView = m_d->clickTransform.map(movingPoint);
+
+        const QPointF projNormVector =
+            KisAlgebra2D::normalize(movingPointInView - staticPointInView);
+
+        const qreal projLength =
+            KisAlgebra2D::dotProduct(mousePos - staticPointInView, projNormVector);
+
+        const QPointF targetMovingPointInView = staticPointInView + projNormVector * projLength;
+
         // override scale static point if it is locked
         if ((m_d->currentArgs.transformAroundRotationCenter() ^ altModifierActive) &&
             !qFuzzyCompare(anchorPoint.x(), movingPoint.x())) {
 
             staticPoint = anchorPoint;
+            staticPointInView = m_d->clickTransform.map(staticPoint);
         }
 
-        QPointF mouseImagePos = m_d->transform.inverted().map(mousePos);
-
-        qreal sign = mouseImagePos.x() <= staticPoint.x() ? -extraSign : extraSign;
-        m_d->currentArgs.setScaleX(sign * m_d->currentArgs.scaleX());
-
-        QPointF staticPointInView = m_d->transform.map(staticPoint);
-
-        qreal dist = kisDistance(staticPointInView, mousePos);
-
         GSL::ScaleResult1D result =
             GSL::calculateScaleX(m_d->currentArgs,
                                  staticPoint,
                                  staticPointInView,
                                  movingPoint,
-                                 dist);
+                                 targetMovingPointInView);
+
+        if (!result.isValid) {
+            break;
+        }
 
         if (shiftModifierActive  ||  m_d->currentArgs.keepAspectRatio()) {
             qreal aspectRatio = m_d->clickArgs.scaleY() / m_d->clickArgs.scaleX();
@@ -605,30 +620,40 @@ void KisFreeTransformStrategy::continuePrimaryAction(const QPointF &mousePos,
             staticPoint = anchorPoint;
         }
 
-        QPointF staticPointInView = m_d->transform.map(staticPoint);
+        QPointF staticPointInView = m_d->clickTransform.map(staticPoint);
         QPointF movingPointInView = mousePos;
 
         if (shiftModifierActive  ||  m_d->currentArgs.keepAspectRatio()) {
-            KisTransformUtils::MatricesPack m(m_d->clickArgs);
-            QTransform t = m.finalTransform();
-
-            QPointF refDiff = t.map(movingPoint) - staticPointInView;
+            QPointF refDiff = m_d->clickTransform.map(movingPoint) - staticPointInView;
             QPointF realDiff = mousePos - staticPointInView;
             realDiff = kisProjectOnVector(refDiff, realDiff);
 
             movingPointInView = staticPointInView + realDiff;
         }
 
+        const bool isAffine =
+            qFuzzyIsNull(m_d->currentArgs.aX()) &&
+            qFuzzyIsNull(m_d->currentArgs.aY());
+
         GSL::ScaleResult2D result =
-            GSL::calculateScale2D(m_d->currentArgs,
-                                  staticPoint,
-                                  staticPointInView,
-                                  movingPoint,
-                                  movingPointInView);
+                !isAffine ?
+                    GSL::calculateScale2D(m_d->currentArgs,
+                                          staticPoint,
+                                          staticPointInView,
+                                          movingPoint,
+                                          movingPointInView) :
+                    GSL::calculateScale2DAffine(m_d->currentArgs,
+                                                staticPoint,
+                                                staticPointInView,
+                                                movingPoint,
+                                                movingPointInView);
+
+        if (result.isValid) {
+            m_d->currentArgs.setScaleX(result.scaleX);
+            m_d->currentArgs.setScaleY(result.scaleY);
+            m_d->currentArgs.setTransformedCenter(result.transformedCenter);
+        }
 
-        m_d->currentArgs.setScaleX(result.scaleX);
-        m_d->currentArgs.setScaleY(result.scaleY);
-        m_d->currentArgs.setTransformedCenter(result.transformedCenter);
         break;
     }
     case MOVECENTER: {
@@ -653,6 +678,8 @@ void KisFreeTransformStrategy::continuePrimaryAction(const QPointF &mousePos,
     case BOTTOMSHEAR: {
         KisTransformUtils::MatricesPack m(m_d->clickArgs);
 
+        QPointF oldStaticPoint = m.finalTransform().map(m_d->clickArgs.originalCenter() + m_d->clickArgs.rotationCenterOffset());
+
         QTransform backwardT = (m.S * m.projectedP).inverted();
         QPointF diff = backwardT.map(mousePos - m_d->clickPos);
 
@@ -664,6 +691,11 @@ void KisFreeTransformStrategy::continuePrimaryAction(const QPointF &mousePos,
 
         // calculate the new shearX factor
         m_d->currentArgs.setShearX(sign * dx / m_d->currentArgs.scaleY() / m_d->transaction.originalHalfHeight()); // calculate the new shearX factor
+
+        KisTransformUtils::MatricesPack currentM(m_d->currentArgs);
+        QTransform t = currentM.finalTransform();
+        QPointF newStaticPoint = t.map(m_d->clickArgs.originalCenter() + m_d->clickArgs.rotationCenterOffset());
+        m_d->currentArgs.setTransformedCenter(m_d->currentArgs.transformedCenter() + oldStaticPoint - newStaticPoint);
         break;
     }
 
@@ -671,6 +703,8 @@ void KisFreeTransformStrategy::continuePrimaryAction(const QPointF &mousePos,
     case RIGHTSHEAR: {
         KisTransformUtils::MatricesPack m(m_d->clickArgs);
 
+        QPointF oldStaticPoint = m.finalTransform().map(m_d->clickArgs.originalCenter() + m_d->clickArgs.rotationCenterOffset());
+
         QTransform backwardT = (m.S * m.projectedP).inverted();
         QPointF diff = backwardT.map(mousePos - m_d->clickPos);
 
@@ -682,6 +716,11 @@ void KisFreeTransformStrategy::continuePrimaryAction(const QPointF &mousePos,
 
         // calculate the new shearY factor
         m_d->currentArgs.setShearY(sign * dy / m_d->clickArgs.scaleX() / m_d->transaction.originalHalfWidth());
+
+        KisTransformUtils::MatricesPack currentM(m_d->currentArgs);
+        QTransform t = currentM.finalTransform();
+        QPointF newStaticPoint = t.map(m_d->clickArgs.originalCenter() + m_d->clickArgs.rotationCenterOffset());
+        m_d->currentArgs.setTransformedCenter(m_d->currentArgs.transformedCenter() + oldStaticPoint - newStaticPoint);
         break;
     }
     }
