@@ -70,6 +70,8 @@
 
 #include <KoShapeTransformCommand.h>
 
+#include "kis_action_registry.h"
+
 #include "widgets/kis_progress_widget.h"
 
 #include "kis_transform_utils.h"
@@ -78,6 +80,7 @@
 #include "kis_liquify_transform_strategy.h"
 #include "kis_free_transform_strategy.h"
 #include "kis_perspective_transform_strategy.h"
+#include "kis_mesh_transform_strategy.h"
 
 #include "kis_transform_mask.h"
 #include "kis_transform_mask_adapter.h"
@@ -103,6 +106,10 @@ KisToolTransform::KisToolTransform(KoCanvasBase * canvas)
         new KisLiquifyTransformStrategy(
             dynamic_cast<KisCanvas2*>(canvas)->coordinatesConverter(),
             m_currentArgs, m_transaction, canvas->resourceManager()))
+    , m_meshStrategy(
+        new KisMeshTransformStrategy(
+            dynamic_cast<KisCanvas2*>(canvas)->coordinatesConverter(),
+            m_currentArgs, m_transaction))
     , m_freeStrategy(
         new KisFreeTransformStrategy(
             dynamic_cast<KisCanvas2*>(canvas)->coordinatesConverter(),
@@ -123,6 +130,7 @@ KisToolTransform::KisToolTransform(KoCanvasBase * canvas)
 
     warpAction = new KisAction(i18n("Warp"));
     liquifyAction = new KisAction(i18n("Liquify"));
+    meshAction = new KisAction(i18n("Mesh"));
     cageAction = new KisAction(i18n("Cage"));
     freeTransformAction = new KisAction(i18n("Free"));
     perspectiveAction = new KisAction(i18n("Perspective"));
@@ -148,6 +156,7 @@ KisToolTransform::KisToolTransform(KoCanvasBase * canvas)
     connect(m_freeStrategy.data(), SIGNAL(requestShowImageTooBig(bool)), SLOT(imageTooBigRequested(bool)));
     connect(m_perspectiveStrategy.data(), SIGNAL(requestCanvasUpdate()), SLOT(canvasUpdateRequested()));
     connect(m_perspectiveStrategy.data(), SIGNAL(requestShowImageTooBig(bool)), SLOT(imageTooBigRequested(bool)));
+    connect(m_meshStrategy.data(), SIGNAL(requestCanvasUpdate()), SLOT(canvasUpdateRequested()));
 
     connect(&m_changesTracker, SIGNAL(sigConfigChanged(KisToolChangesTrackerDataSP)),
             this, SLOT(slotTrackerChangedConfig(KisToolChangesTrackerDataSP)));
@@ -196,6 +205,8 @@ KisTransformStrategyBase* KisToolTransform::currentStrategy() const
         return m_cageStrategy.data();
     } else if (m_currentArgs.mode() == ToolTransformArgs::LIQUIFY) {
         return m_liquifyStrategy.data();
+    } else if (m_currentArgs.mode() == ToolTransformArgs::MESH) {
+        return m_meshStrategy.data();
     } else /* if (m_currentArgs.mode() == ToolTransformArgs::PERSPECTIVE_4POINT) */ {
         return m_perspectiveStrategy.data();
     }
@@ -212,7 +223,6 @@ void KisToolTransform::paint(QPainter& gc, const KoViewConverter &converter)
         m_refRect = newRefRect;
         currentStrategy()->externalConfigChanged();
     }
-
     currentStrategy()->paint(gc);
 
 
@@ -273,7 +283,7 @@ void KisToolTransform::beginActionImpl(KoPointerEvent *event, bool usePrimaryAct
     }
 
     if (!m_strokeId) {
-        startStroke(m_currentArgs.mode(), false);
+        startStroke(m_currentArgs.mode(), action == KisTool::ChangeSize);
     } else if (m_transaction.rootNode()) {
         bool result = false;
 
@@ -351,6 +361,7 @@ QMenu*  KisToolTransform::popupActionsMenu()
         m_contextMenu->addAction(warpAction);
         m_contextMenu->addAction(cageAction);
         m_contextMenu->addAction(liquifyAction);
+        m_contextMenu->addAction(meshAction);
 
         // extra options if free transform is selected
         if (transformMode() == FreeTransformMode) {
@@ -471,6 +482,9 @@ KisToolTransform::TransformToolMode KisToolTransform::transformMode() const
     case ToolTransformArgs::PERSPECTIVE_4POINT:
         mode = PerspectiveTransformMode;
         break;
+    case ToolTransformArgs::MESH:
+        mode = MeshTransformMode;
+        break;
     default:
         KIS_ASSERT_RECOVER_NOOP(0 && "unexpected transform mode");
     }
@@ -567,6 +581,9 @@ void KisToolTransform::setTransformMode(KisToolTransform::TransformToolMode newM
     case PerspectiveTransformMode:
         mode = ToolTransformArgs::PERSPECTIVE_4POINT;
         break;
+    case MeshTransformMode:
+        mode = ToolTransformArgs::MESH;
+        break;
     default:
         KIS_ASSERT_RECOVER_NOOP(0 && "unexpected transform mode");
     }
@@ -582,7 +599,10 @@ void KisToolTransform::setTransformMode(KisToolTransform::TransformToolMode newM
             m_optionsWidget->slotSetLiquifyModeButtonClicked( true );
         } else if( newMode == PerspectiveTransformMode ) {
             m_optionsWidget->slotSetPerspectiveModeButtonClicked( true );
+        } else if( newMode == MeshTransformMode ) {
+            m_optionsWidget->slotSetMeshModeButtonClicked( true );
         }
+
         emit transformModeChanged();
     }
 }
@@ -687,11 +707,28 @@ void KisToolTransform::initThumbnailImage(KisPaintDeviceSP previewDevice)
     m_warpStrategy->setThumbnailImage(origImg, thumbToImageTransform);
     m_cageStrategy->setThumbnailImage(origImg, thumbToImageTransform);
     m_liquifyStrategy->setThumbnailImage(origImg, thumbToImageTransform);
+    m_meshStrategy->setThumbnailImage(origImg, thumbToImageTransform);
 }
 
 void KisToolTransform::activate(ToolActivation toolActivation, const QSet<KoShape*> &shapes)
 {
     KisTool::activate(toolActivation, shapes);
+    m_actionConnections.addConnection(action("movetool-move-up"), SIGNAL(triggered(bool)),
+                                      this, SLOT(slotMoveDiscreteUp()));
+    m_actionConnections.addConnection(action("movetool-move-up-more"), SIGNAL(triggered(bool)),
+                                      this, SLOT(slotMoveDiscreteUpMore()));
+    m_actionConnections.addConnection(action("movetool-move-down"), SIGNAL(triggered(bool)),
+                                      this, SLOT(slotMoveDiscreteDown()));
+    m_actionConnections.addConnection(action("movetool-move-down-more"), SIGNAL(triggered(bool)),
+                                      this, SLOT(slotMoveDiscreteDownMore()));
+    m_actionConnections.addConnection(action("movetool-move-left"), SIGNAL(triggered(bool)),
+                                      this, SLOT(slotMoveDiscreteLeft()));
+    m_actionConnections.addConnection(action("movetool-move-left-more"), SIGNAL(triggered(bool)),
+                                      this, SLOT(slotMoveDiscreteLeftMore()));
+    m_actionConnections.addConnection(action("movetool-move-right"), SIGNAL(triggered(bool)),
+                                      this, SLOT(slotMoveDiscreteRight()));
+    m_actionConnections.addConnection(action("movetool-move-right-more"), SIGNAL(triggered(bool)),
+                                      this, SLOT(slotMoveDiscreteRightMore()));
 
     if (currentNode()) {
         m_transaction = TransformTransactionProperties(QRectF(), &m_currentArgs, KisNodeSP(), {});
@@ -704,6 +741,7 @@ void KisToolTransform::deactivate()
 {
     endStroke();
     m_canvas->updateCanvas();
+    m_actionConnections.clear();
     KisTool::deactivate();
 }
 
@@ -728,7 +766,7 @@ void KisToolTransform::requestStrokeCancellation()
     if (!m_transaction.rootNode() || m_currentArgs.isIdentity()) {
         cancelStroke();
     } else {
-        slotResetTransform();
+        slotCancelTransform();
     }
 }
 
@@ -751,12 +789,21 @@ void KisToolTransform::startStroke(ToolTransformArgs::TransformMode mode, bool f
         currentNode->inherits("KisCloneLayer")) {
 
         KisCanvas2 *kisCanvas = dynamic_cast<KisCanvas2*>(canvas());
-        kisCanvas->viewManager()->
-            showFloatingMessage(
-                i18nc("floating message in transformation tool",
-                      "Layer type cannot use the transform tool"),
-                 koIcon("object-locked"), 4000, KisFloatingMessage::High);
 
+        if(currentNode->inherits("KisColorizeMask")){
+            kisCanvas->viewManager()->
+                showFloatingMessage(
+                    i18nc("floating message in transformation tool",
+                          "Layer type cannot use the transform tool"),
+                    koIcon("object-locked"), 4000, KisFloatingMessage::High);
+        }
+        else{
+            kisCanvas->viewManager()->
+                showFloatingMessage(
+                    i18nc("floating message in transformation tool",
+                          "Layer type cannot use the transform tool. Use transform mask instead."),
+                    koIcon("object-locked"), 4000, KisFloatingMessage::High);
+        }
         return;
     }
 
@@ -788,7 +835,7 @@ void KisToolTransform::startStroke(ToolTransformArgs::TransformMode mode, bool f
 
     // save unique identifier of the stroke so we could
     // recognize it when sigTransactionGenerated() is
-    // recieved (theoretically, the user can start two
+    // received (theoretically, the user can start two
     // strokes at the same time, if he is quick enough)
     m_strokeStrategyCookie = strategy;
     m_strokeId = image()->startStroke(strategy);
@@ -819,8 +866,11 @@ void KisToolTransform::slotTransactionGenerated(TransformTransactionProperties t
 {
     if (!m_strokeId || strokeStrategyCookie != m_strokeStrategyCookie) return;
 
-    if (transaction.transformedNodes().isEmpty()) {
+    if (transaction.transformedNodes().isEmpty() ||
+        transaction.originalRect().isEmpty()) {
+
         KisCanvas2 *kisCanvas = dynamic_cast<KisCanvas2*>(canvas());
+        KIS_ASSERT(kisCanvas);
         kisCanvas->viewManager()->
             showFloatingMessage(
                 i18nc("floating message in transformation tool",
@@ -942,8 +992,11 @@ QWidget* KisToolTransform::createOptionWidget()
     connect(m_optionsWidget, SIGNAL(sigApplyTransform()),
             this, SLOT(slotApplyTransform()));
 
-    connect(m_optionsWidget, SIGNAL(sigResetTransform()),
-            this, SLOT(slotResetTransform()));
+    connect(m_optionsWidget, SIGNAL(sigResetTransform(ToolTransformArgs::TransformMode)),
+            this, SLOT(slotResetTransform(ToolTransformArgs::TransformMode)));
+
+    connect(m_optionsWidget, SIGNAL(sigCancelTransform()),
+            this, SLOT(slotCancelTransform()));
 
     connect(m_optionsWidget, SIGNAL(sigRestartTransform()),
             this, SLOT(slotRestartTransform()));
@@ -962,10 +1015,11 @@ QWidget* KisToolTransform::createOptionWidget()
     connect(perspectiveAction, SIGNAL(triggered(bool)), this, SLOT(slotUpdateToPerspectiveType()));
     connect(freeTransformAction, SIGNAL(triggered(bool)), this, SLOT(slotUpdateToFreeTransformType()));
     connect(liquifyAction, SIGNAL(triggered(bool)), this, SLOT(slotUpdateToLiquifyType()));
+    connect(meshAction, SIGNAL(triggered(bool)), this, SLOT(slotUpdateToMeshType()));
     connect(cageAction, SIGNAL(triggered(bool)), this, SLOT(slotUpdateToCageType()));
 
     connect(applyTransformation, SIGNAL(triggered(bool)), this, SLOT(slotApplyTransform()));
-    connect(resetTransformation, SIGNAL(triggered(bool)), this, SLOT(slotResetTransform()));
+    connect(resetTransformation, SIGNAL(triggered(bool)), this, SLOT(slotCancelTransform()));
 
 
     updateOptionWidget();
@@ -1015,8 +1069,16 @@ void KisToolTransform::slotApplyTransform()
     QApplication::restoreOverrideCursor();
 }
 
-void KisToolTransform::slotResetTransform()
+void KisToolTransform::slotResetTransform(ToolTransformArgs::TransformMode mode)
 {
+    ToolTransformArgs *config = m_transaction.currentConfig();
+    const ToolTransformArgs::TransformMode previousMode = config->mode();
+    config->setMode(mode);
+
+    if (mode == ToolTransformArgs::WARP) {
+        config->setWarpCalculation(KisWarpTransformWorker::WarpCalculation::GRID);
+    }
+
     if (!m_strokeId || !m_transaction.rootNode()) return;
 
     if (m_currentArgs.continuedTransform()) {
@@ -1035,23 +1097,37 @@ void KisToolTransform::slotResetTransform()
 
         if (transformDiffers &&
             m_currentArgs.continuedTransform()->mode() == savedMode) {
+
             m_currentArgs.restoreContinuedState();
             initGuiAfterTransformMode();
             slotEditingFinished();
 
         } else {
-            KisNodeSP root = m_transaction.rootNode() ? m_transaction.rootNode() : image()->root();
             cancelStroke();
             startStroke(savedMode, true);
 
             KIS_ASSERT_RECOVER_NOOP(!m_currentArgs.continuedTransform());
         }
     } else {
-        initTransformMode(m_currentArgs.mode());
-        slotEditingFinished();
+        if (!TransformStrokeStrategy::shouldRestartStrokeOnModeChange(previousMode,
+                                                                      m_currentArgs.mode(),
+                                                                      m_transaction.transformedNodes())) {
+            initTransformMode(m_currentArgs.mode());
+            slotEditingFinished();
+
+        } else {
+            cancelStroke();
+            startStroke(m_currentArgs.mode(), true);
+
+        }
     }
 }
 
+void KisToolTransform::slotCancelTransform()
+{
+    slotResetTransform(m_transaction.currentConfig()->mode());
+}
+
 void KisToolTransform::slotRestartTransform()
 {
     if (!m_strokeId || !m_transaction.rootNode()) return;
@@ -1069,6 +1145,46 @@ void KisToolTransform::slotEditingFinished()
     commitChanges();
 }
 
+void KisToolTransform::slotMoveDiscreteUp()
+{
+    setTranslateY(translateY()-1.0);
+}
+
+void KisToolTransform::slotMoveDiscreteUpMore()
+{
+    setTranslateY(translateY()-10.0);
+}
+
+void KisToolTransform::slotMoveDiscreteDown()
+{
+    setTranslateY(translateY()+1.0);
+}
+
+void KisToolTransform::slotMoveDiscreteDownMore()
+{
+    setTranslateY(translateY()+10.0);
+}
+
+void KisToolTransform::slotMoveDiscreteLeft()
+{
+    setTranslateX(translateX()-1.0);
+}
+
+void KisToolTransform::slotMoveDiscreteLeftMore()
+{
+    setTranslateX(translateX()-10.0);
+}
+
+void KisToolTransform::slotMoveDiscreteRight()
+{
+    setTranslateX(translateX()+1.0);
+}
+
+void KisToolTransform::slotMoveDiscreteRightMore()
+{
+    setTranslateX(translateX()+10.0);
+}
+
 void KisToolTransform::slotUpdateToWarpType()
 {
     setTransformMode(KisToolTransform::TransformToolMode::WarpTransformMode);
@@ -1089,6 +1205,11 @@ void KisToolTransform::slotUpdateToLiquifyType()
     setTransformMode(KisToolTransform::TransformToolMode::LiquifyTransformMode);
 }
 
+void KisToolTransform::slotUpdateToMeshType()
+{
+    setTransformMode(KisToolTransform::TransformToolMode::MeshTransformMode);
+}
+
 void KisToolTransform::slotUpdateToCageType()
 {
     setTransformMode(KisToolTransform::TransformToolMode::CageTransformMode);
@@ -1123,3 +1244,20 @@ void KisToolTransform::setTranslateX(double translation)
 {
     m_optionsWidget->slotSetTranslateX(translation);
 }
+
+QList<QAction *> KisToolTransformFactory::createActionsImpl()
+{
+    KisActionRegistry *actionRegistry = KisActionRegistry::instance();
+    QList<QAction *> actions = KisToolPaintFactoryBase::createActionsImpl();
+
+    actions << actionRegistry->makeQAction("movetool-move-up");
+    actions << actionRegistry->makeQAction("movetool-move-down");
+    actions << actionRegistry->makeQAction("movetool-move-left");
+    actions << actionRegistry->makeQAction("movetool-move-right");
+    actions << actionRegistry->makeQAction("movetool-move-up-more");
+    actions << actionRegistry->makeQAction("movetool-move-down-more");
+    actions << actionRegistry->makeQAction("movetool-move-left-more");
+    actions << actionRegistry->makeQAction("movetool-move-right-more");
+
+    return actions;
+}
