@@ -684,6 +684,20 @@ void TransformStrokeStrategy::finishStrokeImpl(bool applyTransform, const ToolTr
 
     QVector<KisStrokeJobData *> mutatedJobs;
 
+    /**
+     * We should make the shape layers visible **before** transforming them,
+     * otherwise the setDirty() call issued by KisShapeLayerCavas::repaint()
+     * may be lost.
+     */
+    KritaUtils::addJobBarrier(mutatedJobs, [this, applyTransform]() {
+        Q_FOREACH (KisNodeSP node, m_hiddenProjectionLeaves) {
+            node->projectionLeaf()->setTemporaryHiddenFromRendering(false);
+            if (!applyTransform) {
+                node->setDirty();
+            }
+        }
+    });
+
     if (applyTransform) {
         m_savedTransformArgs = args;
 
@@ -702,13 +716,6 @@ void TransformStrokeStrategy::finishStrokeImpl(bool applyTransform, const ToolTr
             selection->setVisible(true);
         }
 
-        Q_FOREACH (KisNodeSP node, m_hiddenProjectionLeaves) {
-            node->projectionLeaf()->setTemporaryHiddenFromRendering(false);
-            if (!applyTransform) {
-                node->setDirty();
-            }
-        }
-
         if (m_deactivatedOverlaySelectionMask) {
             m_deactivatedOverlaySelectionMask->selection()->setVisible(true);
             m_deactivatedOverlaySelectionMask->setDirty();
